@@ -356,7 +356,7 @@
 
   ;; FIXME this is a lie, of course this can fail, but there's no
   ;; error handling here yet!
-  (define-call ("posix_fork" :c-name "fork") pid-t minusp)
+  (define-call-internally posix-fork "fork" pid-t minusp)
   (defun fork ()
     "Forks the current process, returning 0 in the new process and the PID of
 the child process in the parent. Forking while multiple threads are running is
@@ -908,7 +908,8 @@ not supported."
 (progn
   (export 'time :sb-posix)
   (defun time ()
-    (let ((result (alien-funcall (extern-alien "time"
+    (let ((result (alien-funcall (extern-alien #-64-bit-time "time"
+                                               #+64-bit-time "__time64"
                                                (function time-t (* time-t)))
                                  nil)))
       (if (minusp result)
@@ -916,19 +917,19 @@ not supported."
           result)))
   (export 'utime :sb-posix)
   (defun utime (filename &optional access-time modification-time)
-    (let ((fun (extern-alien #-netbsd "utime" #+netbsd "_utime"
-                             (function int (c-string :not-null t)
-                                       (* alien-utimbuf))))
-          (name (filename filename)))
-      (if (not (and access-time modification-time))
-          (alien-funcall fun name nil)
-          (with-alien ((utimbuf (struct alien-utimbuf)))
-            (setf (slot utimbuf 'actime) (or access-time 0)
-                  (slot utimbuf 'modtime) (or modification-time 0))
-            (let ((result (alien-funcall fun name (alien-sap utimbuf))))
-              (if (minusp result)
-                  (syscall-error 'utime)
-                  result))))))
+    (with-alien ((fun (function int (c-string :not-null t) (* alien-utimbuf))
+                      :extern #-(or 64-bit-time netbsd) "utime" #+netbsd "_utime"
+                      #+64-bit-time "__utime64"))
+      (let ((name (filename filename)))
+        (if (not (and access-time modification-time))
+            (alien-funcall fun name nil)
+            (with-alien ((utimbuf (struct alien-utimbuf)))
+              (setf (slot utimbuf 'actime) (or access-time 0)
+                    (slot utimbuf 'modtime) (or modification-time 0))
+              (let ((result (alien-funcall fun name (alien-sap utimbuf))))
+                (if (minusp result)
+                    (syscall-error 'utime)
+                    result)))))))
   (export 'utimes :sb-posix)
   (defun utimes (filename &optional access-time modification-time)
     (flet ((seconds-and-useconds (time)
@@ -939,23 +940,22 @@ not supported."
              (if (minusp value)
                  (syscall-error 'utimes)
                  value)))
-      (let ((fun (extern-alien #-netbsd "utimes" #+netbsd "sb_utimes"
-                               (function int (c-string :not-null t)
-                                                  (* (array alien-timeval 2)))))
-            (name (filename filename)))
-        (if (not (and access-time modification-time))
-            (maybe-syscall-error (alien-funcall fun name nil))
-            (with-alien ((buf (array alien-timeval 2)))
-              (let ((actime (deref buf 0))
-                    (modtime (deref buf 1)))
-                (setf (values (slot actime 'sec)
-                              (slot actime 'usec))
-                      (seconds-and-useconds (or access-time 0))
-                      (values (slot modtime 'sec)
-                              (slot modtime 'usec))
-                      (seconds-and-useconds (or modification-time 0)))
-                (maybe-syscall-error (alien-funcall fun name
-                                                    (alien-sap buf))))))))))
+      (with-alien ((fun (function int (c-string :not-null t) (* (array alien-timeval 2)))
+                        :extern #-(or netbsd 64-bit-time) "utimes" #+(or netbsd 64-bit-time) "sb_utimes"))
+        (let ((name (filename filename)))
+          (if (not (and access-time modification-time))
+              (maybe-syscall-error (alien-funcall fun name nil))
+              (with-alien ((buf (array alien-timeval 2)))
+                (let ((actime (deref buf 0))
+                      (modtime (deref buf 1)))
+                  (setf (values (slot actime 'sec)
+                                (slot actime 'usec))
+                        (seconds-and-useconds (or access-time 0))
+                        (values (slot modtime 'sec)
+                                (slot modtime 'usec))
+                        (seconds-and-useconds (or modification-time 0)))
+                  (maybe-syscall-error (alien-funcall fun name
+                                                      (alien-sap buf)))))))))))
 
 
 ;;; environment
