@@ -63,6 +63,7 @@
         *print-radix* nil
         *print-vector-length* nil
         *print-circle* nil
+        *print-circle-not-shared* nil
         *print-case* :upcase
         *print-array* t
         *print-gensym* t
@@ -155,9 +156,7 @@
   ;; this to be initialized, so we initialize it right away.
   (show-and-call !random-cold-init)
 
-  ;; We can't do such much as a simple PROCLAIM without this global
-  ;; hash-table (because of WARN-IF-INLINE-FAILED/PROCLAIM)
-  (setf sb-c::*emitted-full-calls* (make-hash-table :test 'equal :synchronized t))
+  (setq sb-c::*compilation-unit* nil) ; its DEFVAR is not processed yet
 
   ;; All sorts of things need INFO and/or (SETF INFO).
   (/show0 "about to SHOW-AND-CALL !GLOBALDB-COLD-INIT")
@@ -168,6 +167,8 @@
   ;; And now *CURRENT-THREAD*
   (sb-thread::init-main-thread)
 
+  (show-and-call !hash-table-cold-init)
+
   ;; not sure why this is needed on some architectures. Dark magic.
   #-linkage-space
   (setf (fdefn-fun (find-or-create-fdefn '%coerce-callable-for-call))
@@ -264,8 +265,7 @@
   (makunbound '*!cold-toplevels*) ; so it gets GC'd
 
   ;; Need the static-space replica of the assembly routine jump vector
-  ;; filled in, and the static space vector of static fdefns.
-  ;; This matters only for code that gets compiled to dynamic space,
+  ;; filled in. This matters only for code that gets compiled to dynamic space,
   ;; so it's OK that it occurs somewhat late in cold-init.
   #+x86-64 (sb-vm::validate-asm-routine-vector)
 
