@@ -616,31 +616,28 @@
 
 #+(and sb-unicode 64-bit little-endian
        (not (or arm64 x86-64))) ;; have true simd definitions
-(defun sb-vm::simd-copy-utf8-to-character-string (requested total-copied start string ibuf)
-  (declare (type index start requested total-copied)
+(defun sb-vm::simd-copy-utf8-to-character-string (start end string ibuf)
+  (declare (type index start end)
            (optimize speed (safety 0)))
   (with-pinned-objects (string)
     (let* ((head (buffer-head ibuf))
            (tail (buffer-tail ibuf))
            (sap (buffer-sap ibuf))
-           (left (- requested total-copied))
+           (left (- end start))
            (result-characters (logand left (- sb-vm:n-word-bytes)))
            (string-bytes (logand (- tail head) (- sb-vm:n-word-bytes)))
            (n (min result-characters string-bytes))
            (repeat (ldb (byte sb-vm:n-word-bits 0) #x0101010101010101))
            (ascii-mask (* 128 repeat))
            (string-sap (vector-sap string))
-           (string-start (truly-the fixnum (* (+ start total-copied) 4))))
+           (string-offset (truly-the fixnum (* start 4))))
       (declare (optimize sb-c::preserve-single-use-debug-variables
                          sb-c::preserve-constants))
       (loop for ibuf-offset from head below (+ head n) by sb-vm:n-word-bytes
-            for string-offset from string-start by (* 4 sb-vm:n-word-bytes)
             do
             (let ((word (sap-ref-word sap ibuf-offset)))
               (when (logtest word ascii-mask)
                 (setf head ibuf-offset)
-                (incf total-copied
-                      (truncate (- string-offset string-start) 4))
                 (return))
               (setf (sap-ref-word string-sap string-offset)
                     (dpb (ldb (byte 8 8) word)
@@ -658,31 +655,30 @@
                     (dpb (ldb (byte 8 56) word)
                          (byte 8 32)
                          (ldb (byte 8 48) word))))
-            finally (incf head n)
-                    (incf total-copied n))
+            (incf string-offset (* 4 sb-vm:n-word-bytes))
+            finally (incf head n))
       (setf (buffer-head ibuf) head)
-      total-copied)))
+      (truly-the index (values (truncate string-offset 4))))))
 
 #+(and sb-unicode 64-bit little-endian)
-(defun sb-vm::simd-copy-utf8-crlf-to-character-string-with-size (requested total-copied start string ibuf size-buffer)
-  (declare (type index start requested total-copied)
+(defun sb-vm::simd-copy-utf8-crlf-to-character-string-with-size (start end string ibuf size-buffer)
+  (declare (type index start end)
            (optimize speed (safety 0)))
   (with-pinned-objects (string size-buffer)
     (let* ((head (buffer-head ibuf))
            (tail (buffer-tail ibuf))
            (sap (buffer-sap ibuf))
-           (n (logand (min (- requested total-copied)
+           (n (logand (min (- end start)
                            (- (- tail head) 1))
                       (- 2)))
            (repeat (ldb (byte 16 0) #x0101010101010101))
            (ascii-mask (* 128 repeat))
            (string-sap (vector-sap string))
            (size-sap (vector-sap size-buffer))
-           (size-offset (+ start total-copied))
-           (string-start (* size-offset 4))
-           (string-offset string-start)
+           (size-offset start)
+           (string-offset (* start 4))
            (end (+ head n)))
-      (declare (index size-offset))
+      (declare (index string-offset size-offset))
       (declare (optimize sb-c::preserve-single-use-debug-variables
                          sb-c::preserve-constants))
       (loop while (< head end)
@@ -712,49 +708,45 @@
                      (incf string-offset 8)
                      (incf size-offset 2)))))
       (setf (buffer-head ibuf) head)
-      (truly-the index (+ total-copied (truncate (- string-offset string-start) 4))))))
+      size-offset)))
 
 #+(and sb-unicode 64-bit little-endian
        (not (or arm64 x86-64))) ;; have true simd definitions
-(defun sb-vm::simd-copy-utf8-to-base-string (requested total-copied start string ibuf)
-  (declare (type index start requested total-copied)
+(defun sb-vm::simd-copy-utf8-to-base-string (start end string ibuf)
+  (declare (type index start end)
            (optimize speed (safety 0)))
   (with-pinned-objects (string)
     (let* ((head (buffer-head ibuf))
            (tail (buffer-tail ibuf))
            (sap (buffer-sap ibuf))
-           (n (logand (min (- requested total-copied)
+           (n (logand (min (- end start)
                            (- tail head))
                       (- sb-vm:n-word-bytes)))
            (repeat (ldb (byte sb-vm:n-word-bits 0) #x0101010101010101))
            (ascii-mask (* 128 repeat))
            (string-sap (vector-sap string))
-           (string-start (+ start total-copied)))
+           (string-offset start))
       (declare (optimize sb-c::preserve-single-use-debug-variables
                          sb-c::preserve-constants))
       (loop for ibuf-offset from head below (+ head n) by sb-vm:n-word-bytes
-            for string-offset from string-start by sb-vm:n-word-bytes
             do
             (let ((word (sap-ref-word sap ibuf-offset)))
               (when (logtest word ascii-mask)
                 (setf head ibuf-offset)
-                (incf total-copied
-                      (- string-offset string-start))
                 (return))
               (setf (sap-ref-word string-sap string-offset) word))
-            finally (incf head n)
-                    (incf total-copied n))
+            (incf string-offset sb-vm:n-word-bytes)
+            finally (incf head n))
       (setf (buffer-head ibuf) head)
-      total-copied)))
+      (truly-the index string-offset))))
 
 (defmacro utf8-char-loop (&key size-buffer
                                crlf
                                (eof t))
   `(do ()
        ((or (= tail head)
-            (= requested total-copied)))
-     (let ((byte (sap-ref-8 sap head))
-           (index (+ start total-copied)))
+            (= index end)))
+     (let ((byte (sap-ref-8 sap head)))
        (setf (aref string index)
              (code-char
               (cond
@@ -837,39 +829,37 @@
                      ,@(when size-buffer
                          `((setf (aref size-buffer index) 4)))
                      (setf head new-head)))))))
-       (incf total-copied))
+       (incf index))
      (setf (buffer-head ibuf) head)))
 
-(defun fd-stream-read-n-characters/utf-8 (stream string size-buffer start requested &aux (total-copied 0))
+(defun fd-stream-read-n-characters/utf-8 (stream string size-buffer start end &aux (index start))
   (declare (type fd-stream stream)
-           (type index start requested total-copied)
+           (type index start end index)
            (type ansi-stream-cin-buffer string)
            (type ansi-stream-csize-buffer size-buffer))
   (when (fd-stream-eof-forced-p stream)
     (setf (fd-stream-eof-forced-p stream) nil)
-    (return-from fd-stream-read-n-characters/utf-8 0))
-  (do ((instead (fd-stream-instead stream))
-       (index (+ start total-copied) (1+ index)))
+    (return-from fd-stream-read-n-characters/utf-8 start))
+  (do ((instead (fd-stream-instead stream)))
       ((= (fill-pointer instead) 0)
        (setf (fd-stream-listen stream) nil))
     (setf (aref string index) (vector-pop instead))
     (setf (aref size-buffer index) 0)
-    (incf total-copied)
-    (when (= requested total-copied)
+    (incf index)
+    (when (= end index)
       (when (= (fill-pointer instead) 0)
         (setf (fd-stream-listen stream) nil))
-      (return-from fd-stream-read-n-characters/utf-8 total-copied)))
+      (return-from fd-stream-read-n-characters/utf-8 index)))
   (block outer
     (do ()
         (())
       #+(and sb-unicode 64-bit little-endian)
-      (let ((new-total (sb-vm::simd-copy-utf8-to-character-string requested total-copied
-                                                                  start string (fd-stream-ibuf stream))))
+      (let ((new-index (sb-vm::simd-copy-utf8-to-character-string index end string (fd-stream-ibuf stream))))
         ;; Make sure to change this 1 whenever
         ;; simd-copy-utf8-to-character-string starts processing more than
         ;; just ascii characters.
-        (fill size-buffer 1 :start (+ start total-copied) :end (+ start new-total))
-        (setf total-copied new-total))
+        (fill size-buffer 1 :start index :end new-index)
+        (setf index new-index))
       (let* ((ibuf (fd-stream-ibuf stream))
              (head (buffer-head ibuf))
              (tail (buffer-tail ibuf))
@@ -877,41 +867,39 @@
         (declare (type index head tail))
         (flet ((decode-break (reason)
                  (setf (buffer-head ibuf) head)
-                 (unless (plusp total-copied)
+                 (unless (> index start)
                    (stream-decoding-error-and-handle stream reason 1))
-                 (return-from outer total-copied)))
+                 (return-from outer index)))
           (utf8-char-loop :size-buffer t
                           :eof nil))
-        (when (or (plusp total-copied)
+        (when (or (> index start)
                   (null (catch 'eof-input-catcher (refill-input-buffer stream))))
-          (return total-copied))))))
+          (return index))))))
 
-(defun fd-stream-read-n-characters/utf-8/crlf (stream string size-buffer start requested &aux (total-copied 0))
+(defun fd-stream-read-n-characters/utf-8/crlf (stream string size-buffer start end &aux (index start))
   (declare (type fd-stream stream)
-           (type index start requested total-copied)
+           (type index start end index)
            (type ansi-stream-cin-buffer string)
            (type ansi-stream-csize-buffer size-buffer))
   (when (fd-stream-eof-forced-p stream)
     (setf (fd-stream-eof-forced-p stream) nil)
-    (return-from fd-stream-read-n-characters/utf-8/crlf 0))
-  (do ((instead (fd-stream-instead stream))
-       (index (+ start total-copied) (1+ index)))
+    (return-from fd-stream-read-n-characters/utf-8/crlf start))
+  (do ((instead (fd-stream-instead stream)))
       ((= (fill-pointer instead) 0)
        (setf (fd-stream-listen stream) nil))
     (setf (aref string index) (vector-pop instead))
     (setf (aref size-buffer index) 0)
-    (incf total-copied)
-    (when (= requested total-copied)
+    (incf index)
+    (when (= end index)
       (when (= (fill-pointer instead) 0)
         (setf (fd-stream-listen stream) nil))
-      (return-from fd-stream-read-n-characters/utf-8/crlf total-copied)))
+      (return-from fd-stream-read-n-characters/utf-8/crlf index)))
   (block outer
     (do ()
         (())
       #+(and sb-unicode 64-bit little-endian)
-      (setf total-copied (sb-vm::simd-copy-utf8-crlf-to-character-string-with-size requested total-copied
-                                                                                   start string (fd-stream-ibuf stream)
-                                                                                   size-buffer))
+      (setf index
+            (sb-vm::simd-copy-utf8-crlf-to-character-string-with-size index end string (fd-stream-ibuf stream) size-buffer))
       (let* ((ibuf (fd-stream-ibuf stream))
              (head (buffer-head ibuf))
              (tail (buffer-tail ibuf))
@@ -919,35 +907,33 @@
         (declare (type index head tail))
         (flet ((decode-break (reason)
                  (setf (buffer-head ibuf) head)
-                 (unless (plusp total-copied)
+                 (unless (> index start)
                    (stream-decoding-error-and-handle stream reason 1))
-                 (return-from outer total-copied)))
+                 (return-from outer index)))
           (utf8-char-loop :size-buffer t :crlf t :eof nil))
-        (when (or (plusp total-copied)
+        (when (or (> index start)
                   (null (catch 'eof-input-catcher (refill-input-buffer stream))))
-          (return total-copied))))))
+          (return index))))))
 
 ;;; Bypass the character buffer, the caller must ensure that it's empty
-(defun fd-stream-read-sequence/utf-8-to-string (stream string start requested &aux (total-copied 0))
+(defun fd-stream-read-sequence/utf-8-to-string (stream string start end &aux (index start))
   (declare (type fd-stream stream)
-           (type index start requested total-copied)
+           (type index start end index)
            (type (simple-array character (*)) string))
   (block outer
     (loop
-     (do ((instead (fd-stream-instead stream))
-          (index (+ start total-copied) (1+ index)))
+     (do ((instead (fd-stream-instead stream)))
          ((= (fill-pointer instead) 0)
           (setf (fd-stream-listen stream) nil))
        (setf (aref string index) (vector-pop instead))
-       (incf total-copied)
-       (when (= requested total-copied)
+       (incf index)
+       (when (= index end)
          (when (= (fill-pointer instead) 0)
            (setf (fd-stream-listen stream) nil))
-         (return total-copied)))
+         (return index)))
      #+(and sb-unicode 64-bit little-endian)
-     (setf total-copied
-           (sb-vm::simd-copy-utf8-to-character-string requested total-copied
-                                                      start string (fd-stream-ibuf stream)))
+     (setf index
+           (sb-vm::simd-copy-utf8-to-character-string index end string (fd-stream-ibuf stream)))
      (let* ((ibuf (fd-stream-ibuf stream))
             (head (buffer-head ibuf))
             (tail (buffer-tail ibuf))
@@ -957,38 +943,36 @@
        (flet ((decode-break (reason)
                 (setf (buffer-head ibuf) head)
                 (when (stream-decoding-error-and-handle stream reason 1)
-                  (return-from outer total-copied))
+                  (return-from outer index))
                 (return)))
          (utf8-char-loop))
-       (when (= requested total-copied)
+       (when (= index end)
          (setf (buffer-head ibuf) head)
-         (return total-copied))
+         (return index))
        (unless (catch 'eof-input-catcher (refill-input-buffer stream))
          (when (or (not incomplete)
                    (stream-decoding-error-and-handle stream (- tail head) 1))
-           (return total-copied)))))))
+           (return index)))))))
 
 #+sb-unicode
-(defun fd-stream-read-sequence/utf-8-to-base-string (stream string start requested &aux (total-copied 0))
+(defun fd-stream-read-sequence/utf-8-to-base-string (stream string start end &aux (index start))
   (declare (type fd-stream stream)
-           (type index start requested total-copied)
+           (type index start end index)
            (type simple-base-string string))
   (block outer
     (loop
-     (do ((instead (fd-stream-instead stream))
-          (index (+ start total-copied) (1+ index)))
+     (do ((instead (fd-stream-instead stream)))
          ((= (fill-pointer instead) 0)
           (setf (fd-stream-listen stream) nil))
        (setf (aref string index) (vector-pop instead))
-       (incf total-copied)
-       (when (= requested total-copied)
+       (incf index)
+       (when (= index end)
          (when (= (fill-pointer instead) 0)
            (setf (fd-stream-listen stream) nil))
-         (return total-copied)))
+         (return index)))
      #+(and sb-unicode 64-bit little-endian)
-     (setf total-copied
-           (sb-vm::simd-copy-utf8-to-base-string requested total-copied
-                                                 start string (fd-stream-ibuf stream)))
+     (setf index
+           (sb-vm::simd-copy-utf8-to-base-string index end string (fd-stream-ibuf stream)))
      (let* ((ibuf (fd-stream-ibuf stream))
             (head (buffer-head ibuf))
             (tail (buffer-tail ibuf))
@@ -998,16 +982,127 @@
        (flet ((decode-break (reason)
                 (setf (buffer-head ibuf) head)
                 (when (stream-decoding-error-and-handle stream reason 1)
-                  (return-from outer total-copied))
+                  (return-from outer index))
                 (return)))
          (utf8-char-loop))
-       (when (= requested total-copied)
+       (when (= index end)
          (setf (buffer-head ibuf) head)
-         (return total-copied))
+         (return index))
        (unless (catch 'eof-input-catcher (refill-input-buffer stream))
          (when (or (not incomplete)
                    (stream-decoding-error-and-handle stream (- tail head) 1))
-           (return total-copied)))))))
+           (return index)))))))
+
+#+(and sb-unicode 64-bit little-endian
+       (not (or arm64 x86-64)))
+(defun sb-vm::simd-copy-character-string-to-utf8 (start end string obuf)
+  (declare (type index start end)
+           (optimize speed (safety 0)))
+  (with-pinned-objects (string)
+    (let* ((tail (buffer-tail obuf))
+           (sap (buffer-sap obuf))
+           (buffer-left (- (buffer-length obuf) tail))
+           (string-left (- end start))
+           (n (logand (min buffer-left string-left) -2))
+           (ascii-mask (ldb (byte 64 0)
+                            (lognot (logior 127 (ash 127 32)))))
+           (string-sap (vector-sap string))
+           (string-offset (truly-the fixnum (* start 4)))
+           (last-newline -1))
+      (declare (optimize sb-c::preserve-single-use-debug-variables
+                         sb-c::preserve-constants))
+      (loop for obuf-offset from tail below (+ tail n) by 2
+            do
+            (let ((word (sap-ref-word string-sap string-offset)))
+              (when (logtest word ascii-mask)
+                (setf tail obuf-offset)
+                (return))
+              (let ((a (ldb (byte 8 0) word))
+                    (b (ash word -24)))
+                (when (= a 10)
+                  (setf last-newline string-offset))
+                (when (= b (ash 10 8))
+                  (setf last-newline (truly-the index (+ string-offset 4))))
+                (incf string-offset 8)
+                (setf (sap-ref-16 sap obuf-offset)
+                      (logior a b))))
+            finally (incf tail n))
+      (setf (buffer-tail obuf) tail)
+      (values (truly-the index (values (truncate string-offset 4)))
+              (ash (truly-the fixnum last-newline) -2)))))
+
+(defun output-bytes/utf-8/lf (stream string start* end)
+  (declare (optimize (sb-c:verify-arg-count 0)))
+  (let ((start (or start* 0))
+        (end (or end (length string)))
+        (last-newline -1))
+    (declare (type index start end))
+    (when (fd-stream-synchronize-output stream)
+      (synchronize-stream-output stream))
+    (unless (<= 0 start end (length string))
+      (sequence-bounding-indices-bad-error string start end))
+    (do ()
+        ((= end start))
+      (let ((obuf (fd-stream-obuf stream)))
+        (string-dispatch (simple-base-string (simple-array character (*)) string)
+                         string
+          #+(and sb-unicode 64-bit little-endian)
+          (when (and (typep string '(simple-array character (*)))
+                     (>= (- end start) 16))
+            (multiple-value-bind (new-start newline)
+                (truly-the (values index fixnum &optional) (sb-vm::simd-copy-character-string-to-utf8 start end string obuf))
+              (setf start new-start)
+              (when (>= newline 0)
+                (setf last-newline newline))))
+          (let ((len (- (buffer-length obuf) 4))
+                (sap (buffer-sap obuf))
+                (tail (buffer-tail obuf)))
+            (declare (type index tail)
+                     (optimize (safety 0)))
+            (block output-nothing
+              (do* ()
+                   ((or (= start end) (>= tail len)))
+                (let* ((|ch| (aref string start))
+                       (bits (char-code |ch|)))
+                  (declare (ignorable bits))
+                  (when (char= |ch| #\Newline)
+                    (setf last-newline start))
+                  (cond ((< bits 128)
+                         (setf (sap-ref-8 sap tail) bits) (incf tail))
+                        ((< bits 2048)
+                         (setf (sap-ref-8 sap tail) (logior 192 (ldb (byte 5 6) bits))
+                               (sap-ref-8 sap (+ 1 tail)) (logior 128 (ldb (byte 6 0) bits)))
+                         (incf tail 2))
+                        ((< bits 65536)
+                         (when (<= 55296 bits 57343)
+                           (external-format-encoding-error stream bits))
+                         (setf (sap-ref-8 sap tail) (logior 224 (ldb (byte 4 12) bits))
+                               (sap-ref-8 sap (+ 1 tail)) (logior 128 (ldb (byte 6 6) bits))
+                               (sap-ref-8 sap (+ 2 tail)) (logior 128 (ldb (byte 6 0) bits)))
+                         (incf tail 3))
+                        (t
+                         (setf (sap-ref-8 sap tail) (logior 240 (ldb (byte 3 18) bits))
+                               (sap-ref-8 sap (+ 1 tail)) (logior 128 (ldb (byte 6 12) bits))
+                               (sap-ref-8 sap (+ 2 tail)) (logior 128 (ldb (byte 6 6) bits))
+                               (sap-ref-8 sap (+ 3 tail)) (logior 128 (ldb (byte 6 0) bits)))
+                         (incf tail 4)))
+                  (setf (buffer-tail obuf) tail)
+                  (incf start)))
+              (go flush))
+            (incf start))))
+     flush
+      (when (< start end)
+        (flush-output-buffer stream)))
+    (ecase (fd-stream-buffering stream)
+      (:full)
+      (:line
+       (when (>= last-newline 0)
+         (flush-output-buffer stream)))
+      (:none
+       (flush-output-buffer stream)))
+    (if (>= last-newline 0)
+        (setf (fd-stream-output-column stream) (- end last-newline 1))
+        (incf (fd-stream-output-column stream) (- end (truly-the index start*))))))
 
 (define-external-format/variable-width (:utf-8 :utf8) t
   #+sb-unicode (code-char #xfffd) #-sb-unicode #\?
@@ -1016,19 +1111,27 @@
           ((< bits #x800) 2)
           ((< bits #x10000) 3)
           (t 4)))
-  (ecase size
-    (1 (setf (sap-ref-8 sap tail) bits))
-    (2 (setf (sap-ref-8 sap tail)       (logior #xc0 (ldb (byte 5 6) bits))
-             (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 0) bits))))
-    (3 (when (<= #xd800 bits #xdfff)
-         (external-format-encoding-error stream bits))
+  (cond
+    ((< bits #x80)
+     (setf (sap-ref-8 sap tail) bits)
+     (incf tail))
+    ((< bits #x800)
+     (setf (sap-ref-8 sap tail)       (logior #xc0 (ldb (byte 5 6) bits))
+           (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 0) bits)))
+     (incf tail 2))
+    ((< bits #x10000)
+     (when (<= #xd800 bits #xdfff)
+       (external-format-encoding-error stream bits))
      (setf (sap-ref-8 sap tail)       (logior #xe0 (ldb (byte 4 12) bits))
            (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 6) bits))
-           (sap-ref-8 sap (+ 2 tail)) (logior #x80 (ldb (byte 6 0) bits))))
-    (4 (setf (sap-ref-8 sap tail)       (logior #xf0 (ldb (byte 3 18) bits))
-             (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 12) bits))
-             (sap-ref-8 sap (+ 2 tail)) (logior #x80 (ldb (byte 6 6) bits))
-             (sap-ref-8 sap (+ 3 tail)) (logior #x80 (ldb (byte 6 0) bits)))))
+           (sap-ref-8 sap (+ 2 tail)) (logior #x80 (ldb (byte 6 0) bits)))
+     (incf tail 3))
+    (t
+     (setf (sap-ref-8 sap tail)       (logior #xf0 (ldb (byte 3 18) bits))
+           (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 12) bits))
+           (sap-ref-8 sap (+ 2 tail)) (logior #x80 (ldb (byte 6 6) bits))
+           (sap-ref-8 sap (+ 3 tail)) (logior #x80 (ldb (byte 6 0) bits)))
+     (incf tail 4)))
   (1 (cond ((< byte #x80) 1)
            ((< byte #xc2) (return-from decode-break-reason 1))
            ((< byte #xe0) 2)
@@ -1064,7 +1167,9 @@
   string->utf8
   #+sb-unicode :base-string-direct-mapping #+sb-unicode t
   :fd-stream-read-n-characters fd-stream-read-n-characters/utf-8
-  :char-encodable-p (let ((bits (char-code |ch|))) (not (<= #xd800 bits #xdfff))))
+  :write-n-bytes-fun output-bytes/utf-8/lf
+  :char-encodable-p (let ((bits (char-code |ch|))) (not (<= #xd800 bits #xdfff)))
+  :handle-size nil)
 
 (define-external-format/variable-width (:utf-8) t
   #+sb-unicode (code-char #xfffd) #-sb-unicode #\?
@@ -1073,19 +1178,27 @@
           ((< bits #x800) 2)
           ((< bits #x10000) 3)
           (t 4)))
-  (ecase size
-    (1 (setf (sap-ref-8 sap tail) (if (= bits 10) 13 bits)))
-    (2 (setf (sap-ref-8 sap tail)       (logior #xc0 (ldb (byte 5 6) bits))
-             (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 0) bits))))
-    (3 (when (<= #xd800 bits #xdfff)
-         (external-format-encoding-error stream bits))
+  (cond
+    ((< bits #x80)
+     (setf (sap-ref-8 sap tail) (if (= bits 10) 13 bits))
+     (incf tail))
+    ((< bits #x800)
+     (setf (sap-ref-8 sap tail)       (logior #xc0 (ldb (byte 5 6) bits))
+           (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 0) bits)))
+     (incf tail 2))
+    ((< bits #x10000)
+     (when (<= #xd800 bits #xdfff)
+       (external-format-encoding-error stream bits))
      (setf (sap-ref-8 sap tail)       (logior #xe0 (ldb (byte 4 12) bits))
            (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 6) bits))
-           (sap-ref-8 sap (+ 2 tail)) (logior #x80 (ldb (byte 6 0) bits))))
-    (4 (setf (sap-ref-8 sap tail)       (logior #xf0 (ldb (byte 3 18) bits))
-             (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 12) bits))
-             (sap-ref-8 sap (+ 2 tail)) (logior #x80 (ldb (byte 6 6) bits))
-             (sap-ref-8 sap (+ 3 tail)) (logior #x80 (ldb (byte 6 0) bits)))))
+           (sap-ref-8 sap (+ 2 tail)) (logior #x80 (ldb (byte 6 0) bits)))
+     (incf tail 3))
+    (t
+     (setf (sap-ref-8 sap tail)       (logior #xf0 (ldb (byte 3 18) bits))
+           (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 12) bits))
+           (sap-ref-8 sap (+ 2 tail)) (logior #x80 (ldb (byte 6 6) bits))
+           (sap-ref-8 sap (+ 3 tail)) (logior #x80 (ldb (byte 6 0) bits)))
+     (incf tail 4)))
   (1 (cond ((< byte #x80) 1)
            ((< byte #xc2) (return-from decode-break-reason 1))
            ((< byte #xe0) 2)
@@ -1120,7 +1233,8 @@
   utf8->string/cr-aref
   string/cr->utf8
   :char-encodable-p (let ((bits (char-code |ch|))) (not (<= #xd800 bits #xdfff)))
-  :newline-variant :cr)
+  :newline-variant :cr
+  :handle-size nil)
 
 (define-external-format/variable-width (:utf-8 :utf8) t
   #+sb-unicode (code-char #xfffd) #-sb-unicode #\?
@@ -1130,21 +1244,31 @@
           ((< bits #x800) 2)
           ((< bits #x10000) 3)
           (t 4)))
-  (ecase size
-    (1 (setf (sap-ref-8 sap tail) bits))
-    (2 (if (= bits 10)
-           (setf (sap-ref-8 sap tail) 13 (sap-ref-8 sap (+ 1 tail)) 10)
-           (setf (sap-ref-8 sap tail)       (logior #xc0 (ldb (byte 5 6) bits))
-                 (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 0) bits)))))
-    (3 (when (<= #xd800 bits #xdfff)
-         (external-format-encoding-error stream bits))
+  (cond
+    ((= bits 10)
+     (setf (sap-ref-8 sap tail) 13
+           (sap-ref-8 sap (+ 1 tail)) 10)
+     (incf tail 2))
+    ((< bits #x80)
+     (setf (sap-ref-8 sap tail) (if (= bits 10) 13 bits))
+     (incf tail))
+    ((< bits #x800)
+     (setf (sap-ref-8 sap tail)       (logior #xc0 (ldb (byte 5 6) bits))
+           (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 0) bits)))
+     (incf tail 2))
+    ((< bits #x10000)
+     (when (<= #xd800 bits #xdfff)
+       (external-format-encoding-error stream bits))
      (setf (sap-ref-8 sap tail)       (logior #xe0 (ldb (byte 4 12) bits))
            (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 6) bits))
-           (sap-ref-8 sap (+ 2 tail)) (logior #x80 (ldb (byte 6 0) bits))))
-    (4 (setf (sap-ref-8 sap tail)       (logior #xf0 (ldb (byte 3 18) bits))
-             (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 12) bits))
-             (sap-ref-8 sap (+ 2 tail)) (logior #x80 (ldb (byte 6 6) bits))
-             (sap-ref-8 sap (+ 3 tail)) (logior #x80 (ldb (byte 6 0) bits)))))
+           (sap-ref-8 sap (+ 2 tail)) (logior #x80 (ldb (byte 6 0) bits)))
+     (incf tail 3))
+    (t
+     (setf (sap-ref-8 sap tail)       (logior #xf0 (ldb (byte 3 18) bits))
+           (sap-ref-8 sap (+ 1 tail)) (logior #x80 (ldb (byte 6 12) bits))
+           (sap-ref-8 sap (+ 2 tail)) (logior #x80 (ldb (byte 6 6) bits))
+           (sap-ref-8 sap (+ 3 tail)) (logior #x80 (ldb (byte 6 0) bits)))
+     (incf tail 4)))
   ((2 1)
    (cond
      ((< byte #x80)
@@ -1189,24 +1313,25 @@
   string/crlf->utf8
   :char-encodable-p (let ((bits (char-code |ch|))) (not (<= #xd800 bits #xdfff)))
   :fd-stream-read-n-characters fd-stream-read-n-characters/utf-8/crlf
-  :newline-variant :crlf)
+  :newline-variant :crlf
+  :handle-size nil)
 
 #+(and sb-unicode 64-bit little-endian
        (not arm64)) ;; have true simd definitions or might be redefined with def-variant
-(defun sb-vm::simd-copy-utf8-crlf-to-base-string (requested total-copied start string ibuf)
-  (declare (type index start requested total-copied)
+(defun sb-vm::simd-copy-utf8-crlf-to-base-string (start end string ibuf)
+  (declare (type index start end)
            (optimize speed (safety 0)))
   (with-pinned-objects (string)
     (let* ((head (buffer-head ibuf))
            (tail (buffer-tail ibuf))
            (sap (buffer-sap ibuf))
-           (n (logand (min (- requested total-copied)
+           (n (logand (min (- end start)
                            (- (- tail head) 1))
                       (- 2)))
            (repeat (ldb (byte 16 0) #x0101010101010101))
            (ascii-mask (* 128 repeat))
            (string-sap (vector-sap string))
-           (string-start (+ start total-copied))
+           (string-start start)
            (string-offset string-start)
            (end (+ head n)))
       (declare (optimize sb-c::preserve-single-use-debug-variables
@@ -1229,25 +1354,24 @@
                      (incf head 2)
                      (incf string-offset 2)))))
       (setf (buffer-head ibuf) head)
-      (truly-the index (+ total-copied (- string-offset string-start))))))
+      (truly-the index string-offset))))
 
 #+(and sb-unicode 64-bit little-endian
        (not arm64)) ;; have true simd definitions or might be redefined with def-variant
-(defun sb-vm::simd-copy-utf8-crlf-to-character-string (requested total-copied start string ibuf)
-  (declare (type index start requested total-copied)
+(defun sb-vm::simd-copy-utf8-crlf-to-character-string (start end string ibuf)
+  (declare (type index start end)
            (optimize speed (safety 0)))
   (with-pinned-objects (string)
     (let* ((head (buffer-head ibuf))
            (tail (buffer-tail ibuf))
            (sap (buffer-sap ibuf))
-           (n (logand (min (- requested total-copied)
+           (n (logand (min (- end start)
                            (- (- tail head) 1))
                       (- 2)))
            (repeat (ldb (byte 16 0) #x0101010101010101))
            (ascii-mask (* 128 repeat))
            (string-sap (vector-sap string))
-           (string-start (* (+ start total-copied) 4))
-           (string-offset string-start)
+           (string-offset (* start 4))
            (end (+ head n)))
       (declare (optimize sb-c::preserve-single-use-debug-variables
                          sb-c::preserve-constants))
@@ -1272,30 +1396,29 @@
                      (incf head 2)
                      (incf string-offset 8)))))
       (setf (buffer-head ibuf) head)
-      (truly-the index (+ total-copied (truncate (- string-offset string-start) 4))))))
+      (truly-the index (values (truncate string-offset 4))))))
 
-(defun fd-stream-read-sequence/utf-8-crlf-to-character-string (stream string start requested &aux (total-copied 0))
+(defun fd-stream-read-sequence/utf-8-crlf-to-character-string (stream string start end)
   (declare (type fd-stream stream)
-           (type index start requested total-copied)
+           (type index start end)
            (type (simple-array character (*)) string))
-  (let (eof)
+  (let (eof
+        (index start))
+    (declare (index index))
     (block outer
       (loop
-       (do ((instead (fd-stream-instead stream))
-            (index (+ start total-copied) (1+ index)))
+       (do ((instead (fd-stream-instead stream)))
            ((= (fill-pointer instead) 0)
             (setf (fd-stream-listen stream) nil))
          (setf (aref string index) (vector-pop instead))
-         (incf total-copied)
-         (when (= requested total-copied)
+         (incf index)
+         (when (= index end)
            (when (= (fill-pointer instead) 0)
              (setf (fd-stream-listen stream) nil))
-           (return total-copied)))
+           (return index)))
        #+(and sb-unicode 64-bit little-endian)
-       (setf total-copied
-             (sb-vm::simd-copy-utf8-crlf-to-character-string requested total-copied
-                                                             start string (fd-stream-ibuf stream)))
-
+       (setf index
+             (sb-vm::simd-copy-utf8-crlf-to-character-string index end string (fd-stream-ibuf stream)))
        (let* ((ibuf (fd-stream-ibuf stream))
               (head (buffer-head ibuf))
               (tail (buffer-tail ibuf))
@@ -1306,41 +1429,41 @@
          (flet ((decode-break (reason)
                   (setf (buffer-head ibuf) head)
                   (when (stream-decoding-error-and-handle stream reason 1)
-                    (return-from outer total-copied))
+                    (return-from outer index))
                   (return)))
            (utf8-char-loop :crlf t))
-         (when (or (= requested total-copied)
+         (when (or (= end index)
                    eof)
            (setf (buffer-head ibuf) head)
-           (return total-copied))
+           (return index))
          (unless (catch 'eof-input-catcher (refill-input-buffer stream))
            (if requested-refill
                (setf eof t)
                (when (or (not incomplete)
                          (stream-decoding-error-and-handle stream (- tail head) 1))
-                 (return total-copied)))))))))
+                 (return index)))))))))
 
-(defun fd-stream-read-sequence/utf-8-crlf-to-base-string (stream string start requested &aux (total-copied 0))
+(defun fd-stream-read-sequence/utf-8-crlf-to-base-string (stream string start end)
   (declare (type fd-stream stream)
-           (type index start requested total-copied)
+           (type index start end)
            (type simple-base-string string))
-  (let (eof)
+  (let (eof
+        (index start))
+    (declare (index index))
     (block outer
       (loop
-       (do ((instead (fd-stream-instead stream))
-            (index (+ start total-copied) (1+ index)))
+       (do ((instead (fd-stream-instead stream)))
            ((= (fill-pointer instead) 0)
             (setf (fd-stream-listen stream) nil))
          (setf (aref string index) (vector-pop instead))
-         (incf total-copied)
-         (when (= requested total-copied)
+         (incf index)
+         (when (= index end)
            (when (= (fill-pointer instead) 0)
              (setf (fd-stream-listen stream) nil))
-           (return total-copied)))
+           (return index)))
        #+(and sb-unicode 64-bit little-endian)
-       (setf total-copied
-             (sb-vm::simd-copy-utf8-crlf-to-base-string requested total-copied
-                                                        start string (fd-stream-ibuf stream)))
+       (setf index
+             (sb-vm::simd-copy-utf8-crlf-to-base-string index end string (fd-stream-ibuf stream)))
        (let* ((ibuf (fd-stream-ibuf stream))
               (head (buffer-head ibuf))
               (tail (buffer-tail ibuf))
@@ -1351,16 +1474,16 @@
          (flet ((decode-break (reason)
                   (setf (buffer-head ibuf) head)
                   (when (stream-decoding-error-and-handle stream reason 1)
-                    (return-from outer total-copied))
+                    (return-from outer index))
                   (return)))
            (utf8-char-loop :crlf t))
-         (when (or (= requested total-copied)
+         (when (or (= end index)
                    eof)
            (setf (buffer-head ibuf) head)
-           (return total-copied))
+           (return index))
          (unless (catch 'eof-input-catcher (refill-input-buffer stream))
            (if requested-refill
                (setf eof t)
                (when (or (not incomplete)
                          (stream-decoding-error-and-handle stream (- tail head) 1))
-                 (return total-copied)))))))))
+                 (return index)))))))))
