@@ -60,6 +60,8 @@
   (def %sin-quick (x))
   (def %sqrt (x))
   (def %log (x))
+  (def %log1p (x))
+  (def %log2 (x))
   (def %exp (x)))
 
 #+(or x86-64 arm-vfp arm64 riscv) ;; for constant folding
@@ -119,7 +121,10 @@
 ;;; integers, and inverted if negative.
 (defun intexp (base power)
   (declare (explicit-check))
-  (cond ((eql base 1)
+  (cond ((and (eql base 10)
+              (typep power '(integer 0 20)))
+         (expt 10 power))
+        ((eql base 1)
          base)
         ((eql base -1)
          (if (evenp power)
@@ -149,15 +154,45 @@
          (/ (intexp base (- power))))
         ((eql base 2)
          (ash 1 power))
+        ((not (fixnump power))
+         (error "Exponent too large to fit into memory: ~a" power))
         (t
-         (do* ((base base)
-               (power power)
-               (nextn (ash power -1) (ash power -1))
-               (total (if (oddp power) base 1)
-                      (if (oddp power) (* base total) total)))
-              ((zerop nextn) total)
-           (setq base (* base base))
-           (setq power nextn)))))
+         (prog ((base base)
+                (total (if (oddp power) base 1))
+                (power (ash power -1)))
+            (unless (typep base '#1=(integer #.(- (isqrt most-positive-fixnum)) #.(isqrt most-positive-fixnum)))
+              (go slow1))
+          fast
+            (when (zerop power)
+              (return total))
+            (setf base (* base base))
+            (unless (typep base '#1#)
+              (go slow2))
+            (setf total
+                  (if (oddp power)
+                      (* base (truly-the #1# total))
+                      total)
+                  power (truly-the fixnum (ash power -1)))
+            (go fast)
+          slow1
+            (when (zerop power)
+              (return total))
+            (setf base (* base base))
+          slow2
+            (setf total
+                  (if (oddp power)
+                      (* base total)
+                      total)
+                  power (truly-the fixnum (ash power -1)))
+            (go slow1)))))
+
+(defun 10expt (power)
+  (declare (explicit-check))
+  (cond ((typep power '(integer 0 20))
+         (expt 10 power))
+        (t
+         (locally (declare (notinline expt))
+           (expt 10 power)))))
 
 ;;; If an integer power of a rational, use INTEXP above. Otherwise, do
 ;;; floating point stuff. If both args are real, we try %POW right
@@ -399,26 +434,67 @@
   (if base-p
       (cond
         ((zerop base)
-         (if (or (typep number 'double-float) (typep base 'double-float))
+         (if (or (typep number '(or double-float (complex double-float)))
+                 (typep base '(or double-float (complex double-float))))
              0.0d0
              0.0f0))
+        ((eql number 0)
+         (error 'division-by-zero :operands (list number base) :operation 'log))
         ((and (typep number '(rational 0))
               (typep base '(rational 0)))
          (coerce (/ (truly-the double-float (log2/nonnegative-rational number))
                     (truly-the double-float (log2/nonnegative-rational base)))
                  'single-float))
-        ((and (typep number 'rational)
-              (typep base 'rational))
-         (coerce (/ (log2/rational number)
-                    (log2/rational base))
-                 '(complex single-float)))
-        ((and (typep number 'rational) (typep base 'double-float))
-         (/ (log2/rational number) (log2/double-float base)))
-        ((and (typep number 'double-float) (typep base 'rational))
-         (/ (log2/double-float number) (log2/rational base)))
         (t
-         (/ (log number) (log base))))
+         (number-dispatch ((number number) (base number))
+           ;; (log <real> <real>)
+           ((rational rational)
+            ;; we excluded real results from rational arguments above
+            (coerce (/ (log2/rational number) (log2/rational base))
+                    '(complex single-float)))
+           ((rational (foreach single-float double-float))
+            (let ((result (/ (log2/rational number) (log2/double-float (coerce base 'double-float)))))
+              (if (floatp result)
+                  (coerce result '(dispatch-type base))
+                  (coerce result '(complex (dispatch-type base))))))
+           (((foreach single-float double-float) rational)
+            (let ((result (/ (log2/double-float (coerce number 'double-float)) (log2/rational base))))
+              (if (floatp result)
+                  (coerce result '(dispatch-type number))
+                  (coerce result '(complex (dispatch-type number))))))
+           ((single-float single-float)
+            (/ (log number) (log base)))
+           ((single-float double-float)
+            (/ (log (coerce number 'double-float)) (log base)))
+           ((double-float single-float)
+            (/ (log number) (log (coerce base 'double-float))))
+           ((double-float double-float)
+            (/ (log number) (log base)))
+           ;; complex single-float result
+           (((foreach rational single-float) (foreach (complex rational) (complex single-float)))
+            (/ (log number) (log base)))
+           (((foreach (complex rational) (complex single-float))
+             (foreach rational single-float (complex rational) (complex single-float)))
+            (/ (log number) (log base)))
+           ;; complex double-float result, from contagion
+           (((foreach double-float (complex double-float))
+             (foreach (complex rational) (complex single-float)))
+            (/ (log number) (log (coerce base '(complex double-float)))))
+           (((foreach (complex rational) (complex single-float))
+             (foreach double-float (complex double-float)))
+            (/ (log (coerce number '(complex double-float))) (log base)))
+           (((foreach rational single-float) (complex double-float))
+            (/ (log (coerce number 'double-float)) (log base)))
+           (((complex double-float) (foreach rational single-float))
+            (/ (log number) (log (coerce base 'double-float))))
+           ;; complex double-float result, no contagion
+           ((double-float (complex double-float))
+            (/ (log number) (log base)))
+           (((complex double-float) (foreach double-float (complex double-float)))
+            (/ (log number) (log base))))))
       (let ((log2e 1.4426950408889634d0))
+        (when (eql number 0)
+          (error 'division-by-zero :operands (list number) :operation 'log))
         (number-dispatch ((number number))
           (((foreach fixnum bignum))
            (if (minusp number)
