@@ -39,13 +39,13 @@
   `(let* ((fun ,fun)
           (self
            ;; a few architectures store the untagged address of the entrypoint in 'self'
-           #+(or x86 x86-64 arm64)
+           #+(or arm64 ppc64 x86 x86-64)
            (%make-lisp-obj
             (truly-the word (+ (get-lisp-obj-address fun)
                                (ash sb-vm:simple-fun-insts-offset sb-vm:word-shift)
                                (- sb-vm:fun-pointer-lowtag))))
            ;; all others store the function itself (what else?) in 'self'
-           #-(or x86 x86-64 arm64) fun))
+           #-(or arm64 ppc64 x86 x86-64) fun))
      (setf (sb-vm::%simple-fun-self fun) self)))
 
 (flet ((fixup (code-obj offset name kind flavor-id real-code-obj callees
@@ -69,7 +69,14 @@
                    (:alien-data-linkage-index (sb-impl::ensure-alien-linkage-index name t))
                    (:foreign (foreign-symbol-address name))
                    (:foreign-dataref (foreign-symbol-address name t))
-                   #+linkage-space (:linkage-cell (ensure-linkage-index name))
+                   #+linkage-space
+                   ((:linkage-cell :linkage-cell-ud)
+                    (let* ((quiet (eq flavor :linkage-cell))
+                           (index (ensure-linkage-index name quiet)))
+                      (unless (permanent-fname-p name) (setq callees (adjoin index callees)))
+                      ;; machine-dependent fixup doesn't want to know which flavor was used
+                      (setq flavor :linkage-cell)
+                      index))
                    (:code-object (get-lisp-obj-address real-code-obj))
                    #+sb-thread (:symbol-tls-index (ensure-symbol-tls-index name))
                    (:layout (get-lisp-obj-address
@@ -83,16 +90,12 @@
                    (:symbol-value (get-lisp-obj-address (symbol-global-value name)))
                    (t (bug "bad fixup flavor ~s" flavor)))
                  kind flavor)
-         (if (and (eq flavor :linkage-cell) (not (permanent-fname-p name)))
-             (adjoin name callees :test 'equal)
-             callees))
-       (finish-fixups (code-obj callees retained-fixups)
+         callees)
+       (finish-fixups (code-obj callees other-fixups)
          (declare (ignorable code-obj callees))
          (setf (sb-vm::%code-fixups code-obj)
-               #-linkage-space retained-fixups
-               #+linkage-space
-               (let ((links (pack-code-fixup-locs (mapcar 'ensure-linkage-index callees))))
-                 (join-varint-streams links retained-fixups)))
+               #+linkage-space (join-varint-streams (pack-code-fixup-locs callees) other-fixups)
+               #-linkage-space other-fixups)
          ;; Assign all SIMPLE-FUN-SELF slots unless #+darwin-jit in which case the simple-funs
          ;; are assigned by jit_memcpy_codeblob()
          #-darwin-jit
@@ -158,17 +161,13 @@
         (or #+darwin-jit
             (make-array (- n-boxed-words sb-vm:code-constants-offset) :initial-element 0)))
        (const-patch-start-index sb-vm:code-constants-offset)
-       ;; Pre-scan for all fdefinitions to ensure their existence, which guarantees that
-       ;; storing them into the boxed words can't can't create an old->young pointer.
-       ;; This is essential since gencgc will miss them when scanning the code header
-       ;; for such tagged pointers.
-       (n-fdefns
-        (do ((count 0)
-             (index const-patch-start-index (1+ index)))
-            ((>= index n-boxed-words) count)
+       ;; Ensure existence of FDEFNs before allocating code. This potentially reduces
+       ;; the number of old->young pointers when assigning boxed words.
+       (nil
+        (do ((index const-patch-start-index (1+ index)))
+            ((>= index n-boxed-words))
           (let ((const (aref constants index)))
             (when (typep const '(cons (eql :fdefinition)))
-              (incf count)
               (setf (second const) (find-or-create-fdefn (second const)))))))
        (retained-fixups (sb-c::pack-fixups-for-reapplication fixup-notes))
        ((code-obj total-nwords)
@@ -182,7 +181,7 @@
        (real-code-obj code-obj))
     (declare (ignorable boxed-data))
     (sb-fasl::with-writable-code-instructions
-        (code-obj total-nwords debug-info n-fdefns n-simple-funs)
+        (code-obj total-nwords debug-info n-simple-funs)
         :copy (%byte-blt bytes 0 (code-instructions code-obj) 0 (length bytes))
         :fixup (setq named-call-fixups
                      (apply-core-fixups code-obj fixup-notes retained-fixups real-code-obj)))
@@ -197,15 +196,11 @@
 
     ;; Don't need code pinned now
     ;; (It will implicitly be pinned on the conservatively scavenged backends)
-    (macrolet ((set-boxed-word (i val &optional fdefnp)
-                 (declare (ignorable fdefnp))
-                 ;; Thankfully we don't mix untagged fdefns with darwin-jit
+    (macrolet ((set-boxed-word (i val)
                  #+darwin-jit
                  `(setf (svref boxed-data (- ,i ,sb-vm:code-constants-offset)) ,val)
                  #-darwin-jit
-                 (if fdefnp
-                     `(set-code-fdefn code-obj ,i ,val)
-                     `(setf (code-header-ref code-obj ,i) ,val))))
+                 `(setf (code-header-ref code-obj ,i) ,val)))
 
       (let* ((entries (ir2-component-entries 2comp))
              (fun-index (length entries)))
@@ -217,17 +212,16 @@
       (do ((index const-patch-start-index (1+ index)))
           ((>= index n-boxed-words))
         (let ((const (aref constants index)))
-          (if (typep const '(cons (eql :fdefinition)))
-              (set-boxed-word index (second const) t)
-              (set-boxed-word index
+          (set-boxed-word index
                 (if (constant-p const)
                     (constant-value const)
                     (destructuring-bind (kind payload) const
                       (ecase kind
+                        (:fdefinition payload)
                         (:entry
                          (the function (gethash (leaf-info payload)
                                                 (core-object-entry-table object))))
-                        (:known-fun (%coerce-name-to-fun payload)))))))))
+                        (:known-fun (%coerce-name-to-fun payload))))))))
 
       #+darwin-jit (assign-code-constants code-obj boxed-data))
 
@@ -258,11 +252,10 @@
 ;;; so that it jumps to a tracing routine and then back again.
 ;;; The code that gets copied is just the tracing wrapper.
 ;;; See the example at COMPILE-FUNOBJ-ENCAPSULATION in ntrace
-#+(or x86 x86-64)
+#+(or ppc64 x86 x86-64)
 (defun copy-code-object (code)
   ;; Must have one simple-fun
   (aver (= (code-n-entries code) 1))
-  (aver (zerop (nth-value 1 (code-header-fdefn-range code))))
   (let* ((nbytes (code-object-size code))
          (boxed (code-header-words code)) ; word count
          (unboxed (- nbytes (ash boxed sb-vm:word-shift))) ; byte count
@@ -303,8 +296,8 @@
 ;;; are 0, so the jump table count is 0.
 ;;; Similar considerations pertain to x86[-64] fixups within the machine code.
 
-(defun code-header/trailer-adjust (code-obj expected-nwords n-fdefns)
-  (declare (ignorable expected-nwords n-fdefns))
+(defun code-header/trailer-adjust (code-obj expected-nwords)
+  (declare (ignorable expected-nwords))
   ;; Serial# shares a word with the jump-table word count,
   ;; so we can't assign serial# until after all raw bytes are copied in.
   ;; Do we need unique IDs on the various strange kind of code blobs? These would
@@ -321,9 +314,6 @@
   (let ((base (sap+ (int-sap (get-lisp-obj-address code-obj)) (- sb-vm:other-pointer-lowtag)))
         (physical-nwords ; upper 4 bytes of the header word
          (ash (get-header-data code-obj) -24)))
-    (setf (sap-ref-32 base (+ (ash sb-vm:code-boxed-size-slot sb-vm:word-shift)
-                              #+little-endian 4))
-          n-fdefns)
     (when (/= physical-nwords expected-nwords)
       ;; Oversized allocation must be exactly 2 words more than requested
       (aver (= (- physical-nwords 2) expected-nwords))
@@ -356,12 +346,3 @@
           (list* code data (nconc (coerce data 'list) sb-vm::*pinned-objects*))))
     (sb-vm::jit-copy-code-constants (get-lisp-obj-address code)
                                     (get-lisp-obj-address data))))
-
-(defun set-code-fdefn (code index fdefn)
-  #+untagged-fdefns
-  (with-pinned-objects (fdefn)
-    (setf (code-header-ref code index)
-          (%make-lisp-obj (logandc2 (get-lisp-obj-address fdefn)
-                                    sb-vm:lowtag-mask))))
-  #-untagged-fdefns
-  (setf (code-header-ref code index) fdefn))
