@@ -442,8 +442,8 @@
 
 ;;; There are 2 symbol flag bits. The placement restrictions on these stipulate:
 ;;; - no conflict with the generation number (byte 3, low 4 bits)
-;;; - avoid #+permgen use of byte bit 7 as the "remembered" bit
-;;; - prefer that the uint32_t package ID be naturally aligned if it matters.
+;;; - avoid #+permgen use of byte 3 bit 7 as the "remembered" bit
+;;; - prefer that the uint16_t package ID be naturally aligned if it matters.
 ;;; Given the above:
 ;;; - for x864: byte indices 1 and 2 for the package, byte index 3 for flags so that
 ;;;   the generation byte can stay where it is
@@ -476,43 +476,3 @@
   (defconstant mixed-region-offset 0)
   (defconstant cons-region-offset (* 3 n-word-bytes))
   (defconstant boxed-region-offset (* 6 n-word-bytes)))
-
-#|
-;; Run this in the SB-VM package once for each target feature combo.
-(defun rewrite-widetag-comments ()
-  (rename-file "src/compiler/generic/early-objdef.lisp" "early-objdef.old")
-  (with-open-file (in "src/compiler/generic/early-objdef.old")
-    (with-open-file (out "src/compiler/generic/early-objdef.lisp"
-                         :direction :output :if-exists :supersede)
-      (let* ((target-features
-              (if (find-package "SB-COLD")
-                  (symbol-value (find-symbol "*FEATURES*" "SB-XC"))
-                  *features*))
-             (feature-bits
-              (+ (if (= n-word-bits 64) 1 0)
-                 (if (member :sb-unicode target-features) 0 2)))
-             (comment-col)
-             (comment-offset (aref #(3 8 12 17) feature-bits))
-             (state 0))
-        (loop
-         (let* ((line (read-line in nil))
-                (trimmed (and line (string-left-trim " " line)))
-                symbol)
-           (unless line
-             (return))
-           (when (and (zerop state) (eql 0 (search "bignum-widetag" trimmed)))
-             (setq state 1 comment-col (position #\; line)))
-           (if (and (= state 1) (plusp (length trimmed))
-                    (alpha-char-p (char trimmed 0))
-                    (boundp (setq symbol (read-from-string trimmed))))
-               (let ((new (make-string (+ comment-col 19)
-                                       :initial-element #\Space)))
-                 (replace new line)
-                 (replace new (format nil "~2,'0X" (symbol-value symbol))
-                          :start1 (+ comment-col comment-offset))
-                 (write-line new out))
-               (progn
-                 (write-line line out)
-                 (if (and (= state 1) (string= line ")"))
-                     (setq state 2))))))))))
-|#
