@@ -38,7 +38,7 @@
   #-sb-xc-host (symbol-value 'default-dynamic-space-size))
 
 ;; By happenstance this is the same as small-space-size.
-(defconstant alien-linkage-table-space-size #x100000)
+(defconstant alien-linkage-space-size #x100000)
 
 ;; Define START/END constants for GC spaces.
 ;; Assumptions:
@@ -60,9 +60,6 @@
           ;; the DEFCONSTANT of the same name, hence the suffix.
           &key ((:dynamic-space-start dynamic-space-start*))
                ((:dynamic-space-size dynamic-space-size*))
-               ;; The immobile-space START parameters should not be used
-               ;; except in forcing discontiguous addresses for testing.
-               ;; And of course, don't use them if unsupported.
                ((:fixedobj-space-start fixedobj-space-start*))
                ((:fixedobj-space-size  fixedobj-space-size*) (* 48 1024 1024))
                ((:text-space-start text-space-start*))
@@ -85,19 +82,18 @@
         ((spaces (append `((read-only ,ro-space-size)
                            #+(and win32 x86-64)
                            (seh-data ,(symbol-value '+backend-page-bytes+) win64-seh-data-addr)
-                           #-immobile-space (alien-linkage-table ,alien-linkage-table-space-size)
+                           #-immobile-space (alien-linkage ,alien-linkage-space-size)
                            ;; safepoint on 64-bit uses a relocatable trap page just below the card mark
                            ;; table, which works nicely assuming a register is wired to the card table
                            #+(and sb-safepoint (not x86-64))
                            ;; Must be just before NIL.
                            (safepoint ,(symbol-value '+backend-page-bytes+))
                            (static ,small-space-size)
-                           (permgen 8388608) ; 8MiB
+                           (permgen 33554432) ; 32MiB
                            #+darwin-jit
                            (static-code ,small-space-size))
                          #+immobile-space
                          `((fixedobj ,fixedobj-space-size*)
-                           (alien-linkage-table ,alien-linkage-table-space-size)
                            (text ,text-space-size*))))
          (ptr small-spaces-start)
          (small-space-forms
@@ -107,7 +103,6 @@
                           (member space '(fixedobj text permgen
                                           #+relocatable-static-space safepoint
                                           #+relocatable-static-space static
-                                          #+immobile-space alien-linkage-table
                                           read-only)))
                         (start ptr)
                         (end (+ ptr size)))
@@ -228,6 +223,7 @@
     *immobile-codeblob-tree* ; for generations 0 through 5 inclusive
     *immobile-codeblob-vector* ; for pseudo-static-generation
     *dynspace-codeblob-tree*
+    *linkage-name-map*
     sb-impl::**finalizer-store**
     sb-impl::*finalizer-rehashlist*
     sb-impl::*finalizers-triggered*
@@ -303,7 +299,7 @@
 |#
 
 (defconstant-eqx common-static-fdefns
-    '(;; This the standard set of assembly routines that need to call into lisp.
+    `(;; This is the standard set of assembly routines that need to call into lisp.
       ;; A few backends add TWO-ARG-/= and others to this, in their {arch}/parms
       two-arg-+
       two-arg--
@@ -312,6 +308,7 @@
       two-arg-<
       two-arg->
       two-arg-=
+      #-linkage-space ,@'(
       eql
       %negate
       ;; These next ones are not called from assembly code, but from lisp.
@@ -337,7 +334,7 @@
       vector-hairy-data-vector-set/check-bounds
       vector-hairy-data-vector-ref/check-bounds
       %ldb
-      sb-kernel:vector-unsigned-byte-8-p)
+      vector-unsigned-byte-8-p))
   #'equalp)
 
 ;;; Refer to the lengthy comment in 'src/runtime/interrupt.h' about
@@ -427,6 +424,8 @@
        (defconstant n-symbol-hash-discard-bits
          (let ((precision (+ 32 n-symbol-hash-prng-bits))) ; total N bits
            (- 64 precision)))
+       (defconstant n-linkage-index-bits (or #+linkage-space 19 0))
+       (defconstant symbol-linkage-index-pos 3) ; low 3 bits are reserved
        (defconstant-eqx sb-impl::symbol-hash-prng-byte
          (byte n-symbol-hash-prng-bits (- 32 n-symbol-hash-prng-bits))
          #'equal))
