@@ -87,14 +87,15 @@
 (defknown %other-pointer-subtype-p (t list) boolean
   (movable foldable flushable always-translatable))
 
+(defknown string-designator-p (t) boolean
+  (movable foldable flushable always-translatable))
+
 ;;; Predicates that don't accept T for the first argument type
 (defknown (float-infinity-p float-nan-p float-infinity-or-nan-p)
   (float) boolean (movable foldable flushable))
 
 ;;;; miscellaneous "sub-primitives"
 
-(defknown pointer-hash (t) fixnum (flushable))
-
 (defknown %sp-string-compare
   (simple-string simple-string index (or null index) index (or null index))
   (values index fixnum)
@@ -122,7 +123,7 @@
 ;;; not only symbols. The value is reliable only if the object is a symbol.
 (defknown hash-as-if-symbol-name (t) symbol-name-hash (flushable movable always-translatable))
 
-(defknown %set-symbol-hash (symbol hash-code)
+(defknown %set-symbol-hash ((and (not null) symbol) hash-code)
   t ())
 
 ;;; SYMBOL-PACKAGE-ID for #+compact-symbol demands a vop which avoids loading
@@ -146,7 +147,7 @@
 
 ;;; Return the length of VECTOR.
 ;;; Ordinary code should prefer to use (LENGTH (THE VECTOR FOO)) instead.
-(defknown vector-length (vector) index (flushable dx-safe))
+(defknown vector-length ((read-only vector)) index (flushable dx-safe))
 
 (defknown vector-sap ((simple-unboxed-array (*))) system-area-pointer
   (flushable))
@@ -191,15 +192,18 @@
 (defknown %array-rank (array) %array-rank
   (flushable))
 
-#+(or x86 x86-64 arm64)
 (defknown (%array-rank= widetag=) (t t) boolean
   (flushable))
 
+(defknown vector-data (array index) (values simple-array index)
+  (flushable))
+
 (defknown simple-array-header-of-rank-p (t %array-rank) boolean
   (flushable))
 (defknown sb-kernel::check-array-shape (simple-array list)
   (simple-array)
   (flushable no-verify-arg-count)
+  :derive-type #'result-type-first-arg
   :result-arg 0)
 
 (defknown (%make-instance %make-instance/mixed) (index) instance
@@ -687,7 +691,7 @@
 
 (defknown %simple-fun-type (function) t (flushable))
 
-#+(or x86 x86-64 arm64) (defknown sb-vm::%closure-callee (function) fixnum (flushable))
+#+(or arm64 ppc64 x86 x86-64) (defknown sb-vm::%closure-callee (function) fixnum (flushable))
 (defknown %closure-fun (function) function (flushable))
 
 (defknown %closure-index-ref (function index) t
@@ -711,9 +715,29 @@
   sb-interpreter::interpreted-fun-prototype (flushable))
 
 
-(defknown %data-vector-and-index (array index)
-                                 (values (simple-array * (*)) index)
-                                 (foldable flushable))
+(defknown %data-vector-and-index
+    (array index)
+    (values (simple-array * (*)) index)
+    (foldable flushable))
+
+(defknown %data-vector-and-index/check-bound (array index)
+    (values (simple-array * (*)) index)
+    (foldable))
+
+(defknown %data-vector-and-index/known
+    (array index)
+    (values array index)
+    (flushable always-translatable))
+
+;;; Checks for and adjusts fill-pointer for vector-pop/push and
+;;; returns the underlying simple data vector.
+(defknown %data-vector-pop
+    (array)
+    (values (simple-array * (*)) index))
+
+(defknown %data-vector-push
+    (array)
+    (values (simple-array * (*)) (or null index)))
 
 (defknown restart-point (t) t ())
 
