@@ -155,10 +155,6 @@
       (> two-arg->)
       (<= two-arg-<=)
       (>= two-arg->=)
-      #+sb-xc-host
-      ,@'((sb-xc:< two-arg-<)
-          (sb-xc:= two-arg-=)
-          (sb-xc:> two-arg->))
       (char-equal two-arg-char-equal)
       (char-greaterp two-arg-char-greaterp)
       (char-lessp two-arg-char-lessp)
@@ -231,6 +227,8 @@
                   (unlink-node if)
                   (%delete-lvar-use combination)
                   (use-lvar combination ref-lvar)
+                  (push "unwrap-predicates" (node-source-path con-ref))
+                  (push "unwrap-predicates" (node-source-path alt-ref))
                   (link-blocks block (node-block next)))))))))))
 
 ;;; Convert function designators to functions in calls to known functions
@@ -255,7 +253,7 @@
                (labels ((translate-two-args (name)
                           (and (eql arg-count 2)
                                (not (fun-lexically-notinline-p name (node-lexenv node)))
-                               (cadr (assoc name *two-arg-functions*))))
+                               (cadr (assoc (uncross name) *two-arg-functions*))))
                         (translate (ref)
                           (let* ((leaf (ref-leaf ref))
                                  (fun-name (and (constant-p leaf)
@@ -294,8 +292,9 @@
 (defun change-full-call (combination new-fun-name)
   (let ((ref (lvar-uses (combination-fun combination))))
     (when (ref-p ref)
-      (setf (combination-fun-info combination)
-            (fun-info-or-lose new-fun-name))
+      (when (combination-fun-info combination)
+        (setf (combination-fun-info combination)
+              (fun-info-or-lose new-fun-name)))
       (change-ref-leaf
        ref
        (find-free-fun new-fun-name ""))
@@ -304,19 +303,44 @@
 (defun rewrite-full-call (combination)
   (let ((combination-name (lvar-fun-name (combination-fun combination) t))
         (args (combination-args combination)))
-    (let ((two-arg (assoc combination-name *two-arg-functions*)))
-      (when (and two-arg
-                 (= (length args) 2)
-                 (not (fun-lexically-notinline-p combination-name
-                                                 (node-lexenv combination))))
-        (destructuring-bind (name two-arg &optional types typed-two-arg) two-arg
-          (declare (ignore name))
-          (when (and types
-                     (loop for arg in args
-                           for type in types
-                           always (csubtypep (lvar-type arg) type)))
-            (setf two-arg typed-two-arg))
-          (change-full-call combination two-arg))))))
+    (cond ((eq (combination-kind combination) :known)
+           (let ((two-arg (assoc (uncross combination-name) *two-arg-functions*)))
+             (when (and two-arg
+                        (= (length args) 2))
+               (destructuring-bind (name two-arg &optional types typed-two-arg) two-arg
+                 (declare (ignore name))
+                 (when (and types
+                            (loop for arg in args
+                                  for type in types
+                                  always (csubtypep (lvar-type arg) type)))
+                   (setf two-arg typed-two-arg))
+                 (change-full-call combination two-arg))))
+           (let ((lvar (node-lvar combination)))
+             (when (or (lvar-single-value-p lvar)
+                       (mv-bind-unused-p lvar 1))
+               (let ((single-value-fun (getf '(truncate sb-kernel::truncate1
+                                               floor sb-kernel::floor1
+                                               ceiling sb-kernel::ceiling1)
+                                             combination-name)))
+                 (when single-value-fun
+                   (unless (cdr args)
+                     (let* ((leaf (find-constant 1))
+                            (ref (make-ref leaf))
+                            (lvar (make-lvar combination)))
+                       (use-lvar ref lvar)
+                       (push ref (leaf-refs leaf))
+                       (insert-ref-before ref combination-name)
+                       (setf (cdr args) (list lvar))))
+                   (change-full-call combination single-value-fun)
+                   (setf (node-derived-type combination)
+                         (make-single-value-type (single-value-type (node-derived-type combination)))))))))
+          ((and (eq (combination-kind combination) :full)
+                (not (fun-lexically-notinline-p combination-name (node-lexenv combination))))
+           (let ((specialized (info :function :specialized-xep combination-name)))
+             (when (and specialized
+                        (= (length args)
+                           (length (first specialized))))
+               (change-full-call combination `(sb-impl::specialized-xep ,combination-name ,@specialized))))))))
 
 ;;; The %other-pointer-subtype-p optimizer in ir2opt combines multiple
 ;;; checks for other-pointer into a single widetag load.
