@@ -881,34 +881,17 @@
 
 ;;; Arrange boxed constants so that all :NAMED-CALL constants are first,
 ;;; then constant leaves, and finally LOAD-TIME-VALUE constants.
-;;; There exist a few reasons for placing all the FDEFNs first:
-;;;  * FDEFNs which are referenced for call (versus for value as in #'FUN)
-;;;    may be stored in the code header as untagged pointers. This benefits PPC64
-;;;    because lowtag subtraction can't be had "for free" due to the architectural
-;;;    requirement that lispword-aligned loads need a displacement that is
-;;;    a multiple of 4, which OTHER-POINTER-LOWTAG does not satisfy.
-;;;  * In the current approach for so-called "static" linking of text-space code,
-;;;    we change code instruction bytes so that they call into a simple-fun
-;;;    directly rather than through an fdefn, but the approach is subject to a
-;;;    data race when redefining an fdefn. It's conceivable that the race can be
-;;;    eliminated by substituting placeholders in the code headers of functions
-;;;    that had static linking performed - so that they see a reference to the
-;;;    callee rather than an fdefn - but in order for that to work, we must
-;;;    distinguish between fdefns that are needed for FDEFN-FUN
-;;;    (via IR2-CONVERT-GLOBAL-VAR) versus those which are present to satisfy
-;;;    a GC invariant and are not otherwise actually used.
-;;;  * Even without the preceding change, undo-static-linkage can avoid
-;;;    scanning code constants that are not FDEFNs.
+;;; It is often useful to scan a blob of code to find just its referenced
+;;; callees without having to sift through all constants. Historically there
+;;; were other uses for the sorting.
 (defun sort-boxed-constants (2comp)
   (let* ((sorted (ir2-component-constants 2comp))
          (unsorted (subseq sorted 1))
          (renumbering)) ; alist of (old . new) indices into constant vector
     (setf (fill-pointer sorted) 0)
     ;; add in fixed overhead
-    (let ((n-entries (length (ir2-component-entries 2comp))))
-      (dotimes (i (+ (* sb-vm:code-slots-per-simple-fun n-entries)
-                     sb-vm:code-constants-offset))
-        (vector-push-extend nil sorted)))
+    (dotimes (i sb-vm:code-constants-offset)
+      (vector-push-extend nil sorted))
     (flet ((scan (pass &aux (old-offset 0))
              (dovector (constant unsorted)
                (incf old-offset)
