@@ -253,8 +253,7 @@
               ;; because liveness depends on pointer tracing without looking at code-fixups.
               (when (and sc
                          (or (not immed)
-                             #+permgen (typep (constant-value constant) 'layout)
-                             #+immobile-space
+                             #+(or immobile-space permgen)
                              (let ((val (constant-value constant)))
                                (or (and (symbolp val) (not (sb-vm:static-symbol-p val)))
                                    (typep val 'layout))))
