@@ -40,6 +40,7 @@
   (:vop-var vop)
   (:temporary (:sc unsigned-reg) val-temp)
   (:gc-barrier 0 1 0)
+  (:ignore name)
   (:generator 1
     (cond #+ubsan
           ((and (eql offset sb-vm:array-fill-pointer-slot) ; half-sized slot
@@ -59,10 +60,7 @@
              (emit-code-page-gengc-barrier object val-temp)
              (emit-store (object-slot-ea object offset lowtag) value val-temp)))
           (t
-           (when barrier
-            (if (eq name '%set-symbol-global-value)
-                (emit-symbol-write-barrier vop object val-temp (vop-nth-arg 1 vop))
-                (emit-gengc-barrier object nil val-temp t)))
+           (when barrier (emit-gengc-barrier object nil val-temp t))
            (emit-store (object-slot-ea object offset lowtag) value val-temp)))))
 
 (defun add-symbol-to-remset (vop symbol)
@@ -108,87 +106,13 @@
   (unless (sc-is symbol immediate)
     (emit-gengc-barrier symbol nil temp newval-tn-ref)))
 
-(define-vop (%set-symbol-global-value)
-  (:args (symbol :scs (descriptor-reg immediate))
-         (value :scs (descriptor-reg any-reg immediate)))
-  (:policy :fast-safe)
-  (:temporary (:sc unsigned-reg) val-temp)
-  (:vop-var vop)
-  (:generator 4
-    (emit-symbol-write-barrier vop symbol val-temp (vop-nth-arg 1 vop))
-    (emit-store (if (sc-is symbol immediate)
-                      (symbol-slot-ea (tn-value symbol) symbol-value-slot)
-                      (object-slot-ea symbol symbol-value-slot other-pointer-lowtag))
-                  value val-temp)))
-
-;;; This does not resolve the TLS-INDEX at load-time, because we don't want to
-;;; waste TLS indices for symbols that may never get thread-locally bound.
-;;; So instead we make a fixup to the address of the 4-byte TLS field in the symbol.
-(defun symbol-tls-index-ea (symbol)
-  (ea (make-fixup (tn-value symbol) :immobile-symbol (- 4 other-pointer-lowtag))))
-
-(defun get-symbol-value-slot-ea (ea-tn symbol)
-  (if (sc-is symbol immediate)
-      (inst mov ea-tn (make-fixup (tn-value symbol) :immobile-symbol
-                                  (- (ash symbol-value-slot word-shift) other-pointer-lowtag)))
-      (inst lea ea-tn (object-slot-ea symbol symbol-value-slot other-pointer-lowtag))))
-
-;;; As implemented for x86-64, SET performs at most one conditional branch and no
-;;; unconditional jump, contrasting with the other common approach of branching around
-;;; a store to either the TLS or the global value, followed by an unconditional jump
-;;; out, and in the middle doing the opposite of whichever store was jumped over.
-;;; (see the arm64, ppc64, and riscv implementations)
-;;; I would surmise that one unconditional branch is preferable.
-;;; Slightly in favor of that claim is that if you run this through a C compiler:
-;;;   void f1(long val, int test, long *a, long *b) {
-;;;       if (test) *a = val; else *b = val;
-;;;   }
-;;; it generates the same code as if you had written "*(test?a:b) = val;"
-;;; So while that's not 100% analagous to this situation here, it does imply
-;;; slight favoritism for branch-free code. It might be possible to use CMOV
-;;; if we don't need to emit the GC barrier such as when storing a fixnum.
-;;; On the other hand, Torvalds has ranted against writing branchless code
-;;; for its own sake (https://yarchive.net/comp/linux/cmov.html)
-;;;
-#+sb-thread
-(define-vop (set)
-  (:args (symbol :scs (descriptor-reg immediate)
-                 ;; Don't need a register load from the constant pool
-                 ;; if we're only going to reference TLS
-                 :load-if
-                 (not (typep (info :variable :wired-tls (known-symbol-use-p vop symbol))
-                             '(or (eql :always-thread-local) integer))))
-         (value :scs (descriptor-reg any-reg immediate)))
-  (:temporary (:sc unsigned-reg) val-temp cell)
-  #+gs-seg (:temporary (:sc unsigned-reg) thread-temp)
-  (:vop-var vop)
-  (:generator 4
-    (let ((known-sym (known-symbol-use-p vop symbol)))
-      (when (typep (info :variable :wired-tls known-sym)
-                   '(or (eql :always-thread-local) integer))
-        ;; Never emit a GC barrier for TLS
-        (return-from set
-          (emit-store (ea (make-fixup known-sym :symbol-tls-index) thread-tn)
-                      value val-temp)))
-      (cond ((info :variable :wired-tls known-sym)
-             ;; The TLS index is arbitrary but known to be nonzero,
-             ;; so we can resolve the displacement of the thread-local value
-             ;; at load-time, saving one instruction over the general case.
-             (inst lea cell (ea (make-fixup known-sym :symbol-tls-index) thread-tn)))
-            (t
-             ;; These MOVs look the same, but when the symbol is immediate, this is
-             ;; a load from an absolute address. Needless to say, the names of these
-             ;; accessor macros are arbitrary - the difference is not very apparent.
-             (inst mov :dword cell (if (sc-is symbol immediate)
-                                       (symbol-tls-index-ea symbol)
-                                       (tls-index-of symbol)))
-             (inst add cell thread-tn))))
-    (inst cmp :qword (ea cell) no-tls-value-marker)
-    (inst jmp :ne STORE)
-    (emit-symbol-write-barrier vop symbol val-temp (vop-nth-arg 1 vop))
-    (get-symbol-value-slot-ea cell symbol)
-    STORE
-    (emit-store (ea cell) value val-temp)))
+;; Return the effective address of the value slot of static SYMBOL.
+(defun static-symbol-value-ea (symbol &optional (byte 0))
+   (ea (+ nil-value
+          (static-symbol-offset symbol)
+          (ash symbol-value-slot word-shift)
+          byte
+          (- other-pointer-lowtag))))
 
 (define-vop (fast-symbol-global-value)
   (:args (object :scs (descriptor-reg immediate)))
@@ -214,207 +138,24 @@
       (inst cmp :byte value unbound-marker-widetag)
       (inst jmp :e err-lab))))
 
-;; Return the DISP field to use in an EA relative to thread-base
-(defun load-time-tls-offset (symbol)
-  (let ((where (info :variable :wired-tls symbol)))
-    (cond ((integerp where) where)
-          (t (make-fixup symbol :symbol-tls-index)))))
+(define-vop (%set-symbol-global-value)
+  (:args (symbol :scs (descriptor-reg immediate))
+         (value :scs (descriptor-reg any-reg immediate)))
+  (:policy :fast-safe)
+  (:temporary (:sc unsigned-reg) val-temp)
+  (:vop-var vop)
+  (:generator 4
+    (emit-symbol-write-barrier vop symbol val-temp (vop-nth-arg 1 vop))
+    (emit-store (if (sc-is symbol immediate)
+                      (symbol-slot-ea (tn-value symbol) symbol-value-slot)
+                      (object-slot-ea symbol symbol-value-slot other-pointer-lowtag))
+      value val-temp)))
 
-(deftransform %compare-and-swap-symbol-value ((symbol old new)
-                                              ((constant-arg symbol) t t))
-  (if (eq (info :variable :kind (sb-c:lvar-value symbol)) :global)
-      `(%cas-symbol-global-value symbol old new)
-      (sb-c::give-up-ir1-transform)))
-
-(defmacro symbol-value-slot-ea (sym)    ; SYM is a TN
-  `(ea (- (* symbol-value-slot n-word-bytes) other-pointer-lowtag)
-       ,sym))
-
-(macrolet ((access-wired-tls-val (sym) ; SYM is a symbol
-             `(thread-tls-ea (load-time-tls-offset ,sym)))
-           (load-oldval ()
-             `(if (sc-is old immediate)
-                  (inst mov rax (encode-value-if-immediate old))
-                  (move rax old))))
-
-(define-vop (%cas-symbol-global-value)
-    (:translate %cas-symbol-global-value)
-    (:args (symbol :scs (descriptor-reg immediate) :to (:result 0))
-           (old :scs (descriptor-reg any-reg constant immediate))
-           (new :scs (descriptor-reg any-reg)))
-    ;; RAX is the temp for computing the card mark, so it has to conflict
-    ;; with OLD and therefore the default lifetime spec is fine
-    (:temporary (:sc descriptor-reg :offset rax-offset :to (:result 0)) rax)
-    (:results (result :scs (descriptor-reg any-reg)))
-    (:policy :fast-safe)
-    (:vop-var vop)
-    (:generator 10
-      (emit-symbol-write-barrier vop symbol rax (vop-nth-arg 2 vop))
-      (load-oldval)
-      (inst cmpxchg :lock (if (sc-is symbol immediate)
-                              (symbol-slot-ea (tn-value symbol) symbol-value-slot)
-                              (symbol-value-slot-ea symbol))
-            new)
-      (move result rax)))
-
-(define-vop (%compare-and-swap-symbol-value)
-    (:translate %compare-and-swap-symbol-value)
-    (:args (symbol :scs (descriptor-reg immediate) :to (:result 0))
-           (old :scs (descriptor-reg any-reg constant immediate) :target rax)
-           (new :scs (descriptor-reg any-reg)))
-    (:temporary (:sc descriptor-reg :offset rax-offset
-                 :from (:argument 1) :to (:result 0)) rax)
-    (:temporary (:sc descriptor-reg :to (:result 0)) cell)
-    #+gs-seg (:temporary (:sc unsigned-reg) thread-temp)
-    (:results (result :scs (descriptor-reg any-reg)))
-    (:policy :fast-safe)
-    (:vop-var vop)
-    (:node-var node)
-    (:generator 15
-    ;; This code has two pathological cases: NO-TLS-VALUE-MARKER
-    ;; or UNBOUND-MARKER as NEW: in either case we would end up
-    ;; doing possible damage with CMPXCHG -- so don't do that!
-    ;; Even worse: don't supply old=NO-TLS-VALUE with a symbol whose
-    ;; tls-index=0, because that would succeed, assigning NEW to each
-    ;; symbol in existence having otherwise no thread-local value.
-      #+sb-thread (progn
-      (inst mov :dword cell (if (sc-is symbol immediate)
-                                (symbol-tls-index-ea symbol)
-                                (tls-index-of symbol)))
-      (inst add cell thread-tn)
-      (inst cmp :qword (ea cell) no-tls-value-marker)
-      (inst jmp :ne CAS))
-      ;; GLOBAL. All logic that follows is for both + and - sb-thread
-      (emit-symbol-write-barrier vop symbol cell (vop-nth-arg 2 vop))
-      (get-symbol-value-slot-ea cell symbol)
-      CAS
-      (load-oldval)
-      (inst cmpxchg :lock (ea cell) new)
-      ;; FIXME: if :ALWAYS-BOUND then elide the BOUNDP check.
-      ;; But we don't accept a constant or immediate, so how to know
-      ;; what symbol is being CASed? I kind of feel like whether to perform
-      ;; the check ought to be supplied as codegen info so that we don't
-      ;; conflate storage-class of an arg with whether we could elide the check.
-      (unless (policy node (= safety 0))
-        (inst cmp :byte rax unbound-marker-widetag)
-        (inst jmp :e (generate-error-code vop 'unbound-symbol-error symbol)))
-      (move result rax)))
-
-  #+sb-thread
-  (progn
-    ;; This code is tested by 'codegen.impure.lisp'
-    (defun emit-symeval (value symbol symbol-ref symbol-reg check-boundp vop)
-      (let* ((known-symbol-p (sc-is symbol constant immediate))
-             (known-symbol (and known-symbol-p (tn-value symbol))))
-        ;; In order from best to worst.
-        (cond
-          ((symbol-always-has-tls-value-p symbol-ref (sb-c::vop-node vop))
-           (setq symbol-reg nil)
-           (inst mov value (access-wired-tls-val known-symbol)))
-          (t
-           (cond
-             ((symbol-always-has-tls-index-p known-symbol) ; e.g. CL:*PRINT-BASE*
-              ;; Known nonzero TLS index, but possibly no per-thread value.
-              ;; The TLS value and global value can be loaded independently.
-              (inst mov value (access-wired-tls-val known-symbol))
-              (when (sc-is symbol constant)
-                (inst mov symbol-reg symbol))) ; = MOV Rxx, [RIP-N]
-
-             (known-symbol-p           ; unknown TLS index, possibly 0
-              (sc-case symbol
-                (immediate
-                 ;; load the TLS index from the symbol. TODO: use [RIP-n] mode
-                 ;; for immobile code to make it automatically relocatable.
-                 (inst mov :dword value
-                       ;; slot index 1/2 is the high half of the header word.
-                       (symbol-slot-ea known-symbol 1/2))
-                 ;; read the TLS value using that index
-                 (inst mov value (thread-tls-ea value)))
-                (constant
-
-                 ;; These reads are inextricably data-dependent
-                 (inst mov symbol-reg symbol) ; = MOV REG, [RIP-N]
-                 (inst mov :dword value (tls-index-of symbol-reg))
-                 (inst mov value (thread-tls-ea value)))))
-
-             (t                      ; SYMBOL-VALUE of a random symbol
-              (inst mov :dword symbol-reg (tls-index-of symbol))
-              (inst mov value (thread-tls-ea symbol-reg))
-              (setq symbol-reg symbol)))
-
-           ;; Load the global value if the TLS value didn't exist
-           (inst cmp :qword value no-tls-value-marker)
-           (inst cmov :e value
-                 (if (and known-symbol-p (sc-is symbol immediate))
-                     (symbol-slot-ea known-symbol symbol-value-slot) ; MOV Rxx, imm32
-                     (symbol-value-slot-ea symbol-reg)))))
-
-        (when check-boundp
-          (assemble ()
-            (inst cmp :byte value unbound-marker-widetag)
-            (let* ((immediatep (sc-is symbol immediate))
-                   (staticp (and immediatep (static-symbol-p known-symbol)))
-                   (*location-context* (make-restart-location RETRY value)))
-              ;; IMMEDIATE sc symbols are not in a register (they are accessed
-              ;; via absolute address), nor are they present in the code header.
-              ;; So emit a MOV just before the INT opcode for such symbols,
-              ;; out of the normal execution path. Most static symbols are
-              ;; DEFCONSTANTs or DEFGLOBALs, so this case is infrequent.
-              (inst jmp :e (generate-error-code+
-                            (if staticp
-                                (lambda ()
-                                  (load-immediate vop symbol symbol-reg)))
-                            vop 'unbound-symbol-error
-                            (if (or (not symbol-reg) (and immediatep (not staticp)))
-                                symbol
-                                symbol-reg))))
-            RETRY))))
-
-  ;; With Symbol-Value, we check that the value isn't the trap object.
-    (define-vop (symbol-value)
-      (:translate symbol-value)
-      (:policy :fast-safe)
-      (:args (symbol :scs (descriptor-reg constant immediate) :to (:result 1)))
-      (:arg-refs symbol-ref)
-      ;; TODO: use no temp if the symbol is known to be thread-local
-      ;; (probably IR1 should go SYMBOL-VALUE -> SYMBOL-TLS-VALUE)
-      (:temporary (:sc descriptor-reg) symbol-reg)
-      (:results (value :scs (descriptor-reg any-reg)))
-      (:vop-var vop)
-      (:save-p :compute-only)
-      (:variant-vars check-boundp)
-      (:variant t)
-      (:generator 9 (emit-symeval value symbol symbol-ref symbol-reg check-boundp vop)))
-
-    (define-vop (fast-symbol-value symbol-value)
-    ;; KLUDGE: not really fast, in fact, because we're going to have to
-    ;; do a full lookup of the thread-local area anyway.  But half of
-    ;; the meaning of FAST-SYMBOL-VALUE is "do not signal an error if
-    ;; unbound", which is used in the implementation of COPY-SYMBOL.  --
-    ;; CSR, 2003-04-22
-      (:policy :fast)
-      (:variant nil)
-      (:variant-cost 5))
-
-    ;; TODO: this vop doesn't see that when (INFO :VARIABLE :KIND) = :GLOBAL
-    ;; there is no need to check the TLS. Probably this is better handled in IR1
-    ;; rather than IR2. It would need a new GLOBAL-BOUNDP function.
-    ;; On the other hand, how many users know that you can declaim
-    ;; a variable GLOBAL without using DEFGLOBAL ?
-    (define-vop (boundp)
-      (:translate boundp)
-      (:policy :fast-safe)
-      (:args (object :scs (descriptor-reg)))
-      (:conditional :ne)
-      (:temporary (:sc unsigned-reg) temp)
-      (:generator 9
-        (inst mov :dword temp (tls-index-of object))
-        (inst mov :qword temp (thread-tls-ea temp))
-        (inst cmp :qword temp no-tls-value-marker)
-        (inst cmov :dword :e temp (symbol-value-slot-ea object))
-        (inst cmp :byte temp unbound-marker-widetag))))
-
-) ; END OF MACROLET
+(defun emit-lea-symbol-value-slot (ea-tn symbol) ; dest, src
+  (if (sc-is symbol immediate)
+      (inst mov ea-tn (make-fixup (tn-value symbol) :immobile-symbol
+                                  (- (ash symbol-value-slot word-shift) other-pointer-lowtag)))
+      (inst lea ea-tn (object-slot-ea symbol symbol-value-slot other-pointer-lowtag))))
 
 #-sb-thread
 (progn
@@ -431,7 +172,55 @@
     (:generator 9
       (inst cmp :byte (object-slot-ea
                  symbol symbol-value-slot other-pointer-lowtag)
-            unbound-marker-widetag))))
+            unbound-marker-widetag)))
+
+  (define-vop (%compare-and-swap-symbol-value)
+    (:translate %compare-and-swap-symbol-value %cas-symbol-global-value)
+    (:args (symbol :scs (descriptor-reg) :to (:result 0))
+           (old :scs (descriptor-reg any-reg constant immediate))
+           (new :scs (descriptor-reg any-reg)))
+    (:temporary (:sc descriptor-reg :offset rax-offset :to (:result 0)) rax)
+    (:results (result :scs (descriptor-reg any-reg)))
+    (:policy :fast-safe)
+    (:vop-var vop)
+    (:node-var node)
+    (:generator 15
+      (emit-symbol-write-barrier vop symbol rax (vop-nth-arg 2 vop))
+      (if (sc-is old immediate)
+          (inst mov rax (encode-value-if-immediate old))
+          (move rax old))
+      (inst cmpxchg :lock (object-slot-ea symbol symbol-value-slot other-pointer-lowtag) new)
+      (unless (policy node (= safety 0))
+        (inst cmp :byte rax unbound-marker-widetag)
+        (inst jmp :e (generate-error-code vop 'unbound-symbol-error symbol)))
+      (move result rax)))
+
+  (define-vop (dynbind)
+    (:args (val :scs (any-reg descriptor-reg))
+           (symbol :scs (descriptor-reg)))
+    (:temporary (:sc unsigned-reg) temp bsp)
+    (:generator 5
+      (load-binding-stack-pointer bsp)
+      (loadw temp symbol symbol-value-slot other-pointer-lowtag)
+      (inst add bsp (* binding-size n-word-bytes))
+      (store-binding-stack-pointer bsp)
+      (storew temp bsp (- binding-value-slot binding-size))
+      (storew symbol bsp (- binding-symbol-slot binding-size))
+      (emit-gengc-barrier symbol nil temp)
+      (storew val symbol symbol-value-slot other-pointer-lowtag)))
+
+  (define-vop (unbind)
+    (:temporary (:sc unsigned-reg) symbol value bsp)
+    (:generator 0
+      (load-binding-stack-pointer bsp)
+      (loadw symbol bsp (- binding-symbol-slot binding-size))
+      (emit-gengc-barrier symbol nil value) ; VALUE is the card-mark temp
+      (loadw value bsp (- binding-value-slot binding-size))
+      (storew value symbol symbol-value-slot other-pointer-lowtag)
+      (storew 0 bsp (- binding-symbol-slot binding-size))
+      (storew 0 bsp (- binding-value-slot binding-size))
+      (inst sub bsp (* binding-size n-word-bytes))
+      (store-binding-stack-pointer bsp))))
 
 (define-vop (symbol-hash)
   (:policy :fast-safe)
@@ -494,10 +283,11 @@
     (gcbar)
     (inst cmp :byte (ea (- other-pointer-lowtag) object) fdefn-widetag)
     (inst jmp :ne SYMBOL)
-    (inst mov :dword (ea (- 2 other-pointer-lowtag) object) index)
+    (inst mov :dword (ea (- 4 other-pointer-lowtag) object) index)
     (inst jmp CELL-SET)
     SYMBOL
-    (inst or :dword (object-slot-ea object symbol-hash-slot other-pointer-lowtag) index)
+    (inst or :dword :lock
+          (object-slot-ea object symbol-hash-slot other-pointer-lowtag) index)
     CELL-SET
     (inst mov (ea linkage-cell) linkage-val)))
 (define-vop (set-fname-fun)
@@ -538,141 +328,6 @@
       (inst jmp :e err-lab))
     RETRY))
 
-;;;; binding and unbinding
-
-;;; BIND -- Establish VAL as a binding for SYMBOL. Save the old value and
-;;; the symbol on the binding stack and stuff the new value into the
-;;; symbol.
-;;; See the "Chapter 9: Specials" of the SBCL Internals Manual.
-
-#+sb-thread
-(progn
-(define-vop (dynbind) ; bind a symbol in a PROGV form
-  (:args (val :scs (any-reg descriptor-reg))
-         (symbol :scs (descriptor-reg)))
-  (:temporary (:sc unsigned-reg :offset rax-offset) tls-index)
-  (:temporary (:sc unsigned-reg) bsp tmp)
-  (:vop-var vop)
-  (:generator 10
-    (load-binding-stack-pointer bsp)
-    (inst mov :dword tls-index (tls-index-of symbol))
-    (inst add bsp (* binding-size n-word-bytes))
-    (store-binding-stack-pointer bsp)
-    (inst test :dword tls-index tls-index)
-    (inst jmp :ne TLS-INDEX-VALID)
-    (inst mov tls-index symbol)
-    (invoke-asm-routine 'call 'alloc-tls-index vop)
-    TLS-INDEX-VALID
-    (inst mov tmp (thread-tls-ea tls-index))
-    (storew tmp bsp (- binding-value-slot binding-size))
-    (storew tls-index bsp (- binding-symbol-slot binding-size))
-    (inst mov (thread-tls-ea tls-index) val)))
-
-(defun bind (bsp symbol tmp)
-  (inst mov bsp (* binding-size n-word-bytes))
-  (inst xadd (thread-slot-ea thread-binding-stack-pointer-slot) bsp)
-  (let* ((tls-index (load-time-tls-offset symbol))
-         (tls-cell (thread-tls-ea tls-index)))
-    ;; Too bad we can't use "XCHG [thread + disp], val" to write new value
-    ;; and read the old value in one step. It will violate the constraints
-    ;; prescribed in the internal documentation on special binding.
-    (inst mov tmp tls-cell)
-    (storew tmp bsp binding-value-slot)
-    ;; Indices are small enough to be written as :DWORDs which avoids
-    ;; a REX prefix if 'bsp' happens to be any of the low 8 registers.
-    (inst mov :dword (ea (ash binding-symbol-slot word-shift) bsp) tls-index)
-    (values tls-cell tmp)))
-
-(define-vop (bind) ; bind a known symbol
-  (:args (val :scs (any-reg descriptor-reg)
-              :load-if (not (let ((imm (encode-value-if-immediate val)))
-                              (or (fixup-p imm)
-                                  (plausible-signed-imm32-operand-p imm))))))
-  (:temporary (:sc unsigned-reg) bsp tmp)
-  (:info symbol)
-  (:generator 10
-    (inst mov :qword (bind bsp symbol tmp) (encode-value-if-immediate val))))
-
-(define-vop (rebind)
-  (:temporary (:sc unsigned-reg) bsp tls-value)
-  (:temporary (:sc descriptor-reg
-               :unused-if (eq (immediate-constant-sc symbol) immediate-sc-number))
-              symbol-reg)
-  (:info symbol)
-  (:node-var node)
-  (:vop-var vop)
-  (:generator 10
-    (multiple-value-bind (tls-cell tls-value) (bind bsp symbol tls-value)
-      (unless (symbol-always-has-tls-value-p symbol node)
-        (unless (eq (tn-kind symbol-reg) :unused)
-          (load-constant vop (emit-constant symbol) symbol-reg))
-        (inst cmp tls-value no-tls-value-marker)
-        (inst cmov :e tls-value
-              (if (eq (tn-kind symbol-reg) :unused)
-                  (symbol-slot-ea symbol symbol-value-slot)
-                  (symbol-value-slot-ea symbol-reg)))
-        (inst mov tls-cell tls-value))))))
-
-#-sb-thread
-(define-vop (dynbind)
-  (:args (val :scs (any-reg descriptor-reg))
-         (symbol :scs (descriptor-reg)))
-  (:temporary (:sc unsigned-reg) temp bsp)
-  (:generator 5
-    (load-binding-stack-pointer bsp)
-    (loadw temp symbol symbol-value-slot other-pointer-lowtag)
-    (inst add bsp (* binding-size n-word-bytes))
-    (store-binding-stack-pointer bsp)
-    (storew temp bsp (- binding-value-slot binding-size))
-    (storew symbol bsp (- binding-symbol-slot binding-size))
-    (emit-gengc-barrier symbol nil temp)
-    (storew val symbol symbol-value-slot other-pointer-lowtag)))
-
-#+sb-thread
-(define-vop (unbind-n)
-  (:temporary (:sc unsigned-reg) temp bsp)
-  (:temporary (:sc complex-double-reg) zero)
-  (:info symbols)
-  (:vop-var vop)
-  (:generator 0
-    (load-binding-stack-pointer bsp)
-    (inst xorpd zero zero)
-    (loop for symbol in symbols
-          for tls-index = (load-time-tls-offset symbol)
-          for tls-cell = (thread-tls-ea tls-index)
-          do
-          #+ultrafutex
-          (when (eq symbol '*current-mutex*)
-            (let ((uncontested (gen-label)))
-              (inst mov temp tls-cell) ; load the current value
-              (inst mov :qword (mutex-slot temp %owner) 0)
-              (inst dec :lock :byte (mutex-slot temp state))
-              (inst jmp :z uncontested) ; if ZF then previous value was 1, no waiters
-              (invoke-asm-routine 'call 'mutex-wake-waiter vop)
-              (emit-label uncontested)))
-
-          (inst sub bsp (* binding-size n-word-bytes))
-
-          ;; Load VALUE from stack, then restore it to the TLS area.
-          (loadw temp bsp binding-value-slot)
-          (inst mov tls-cell temp)
-          ;; Zero out the stack.
-          (inst movapd (ea bsp) zero))
-    (store-binding-stack-pointer bsp)))
-
-#-sb-thread
-(define-vop (unbind)
-  (:temporary (:sc unsigned-reg) symbol value bsp)
-  (:generator 0
-    (load-binding-stack-pointer bsp)
-    (loadw symbol bsp (- binding-symbol-slot binding-size))
-    (emit-gengc-barrier symbol nil value) ; VALUE is the card-mark temp
-    (loadw value bsp (- binding-value-slot binding-size))
-    (storew value symbol symbol-value-slot other-pointer-lowtag)
-    (storew 0 bsp (- binding-symbol-slot binding-size))
-    (storew 0 bsp (- binding-value-slot binding-size))
-    (inst sub bsp (* binding-size n-word-bytes))
-    (store-binding-stack-pointer bsp)))
 
 (defun unbind-to-here (where symbol value bsp
                        zero)
