@@ -95,7 +95,7 @@ copied_realpath(const char *pathname)
      * an absolute path, so we prepend the cwd to relative paths */
     messy = NULL;
     if (pathname[0] != '/') {
-        messy = successful_malloc(PATH_MAX + 1);
+        messy = checked_malloc(PATH_MAX + 1);
         if (getcwd(messy, PATH_MAX + 1) == NULL) {
             free(messy);
             return NULL;
@@ -104,13 +104,19 @@ copied_realpath(const char *pathname)
         snprintf(messy + len, PATH_MAX + 1 - len, "/%s", pathname);
     }
 
-    tidy = successful_malloc(PATH_MAX + 1);
+    tidy = checked_malloc(PATH_MAX + 1);
+#ifdef LISP_FEATURE_ANSI_COMPLIANT_LOAD_TRUENAME
     if (realpath((messy ? messy : pathname), tidy) == NULL) {
         if (messy)
             free(messy);
         free(tidy);
         return NULL;
     }
+#else
+    // I don't know why avoiding realpath shouldn't be the default behavior
+    // but I'm sure some user would complain if it were.
+    strcpy(tidy, (messy ? messy : pathname));
+#endif
 
     if (messy)
         free(messy);
@@ -251,7 +257,7 @@ search_for_executable(const char *argv0)
     if (search == NULL)
         return NULL;
     search = copied_string(search);
-    buf = successful_malloc(PATH_MAX + 1);
+    buf = checked_malloc(PATH_MAX + 1);
     for (start = search; (end = strchr(start, ':')) != NULL; start = end + 1) {
         *end = '\0';
         snprintf(buf, PATH_MAX + 1, "%s/%s", start, argv0);
@@ -354,7 +360,7 @@ char *dir_name(char *path) {
 
     if (slash) {
         int prefixlen = slash - path + 1; // keep the slash in the prefix
-        result = successful_malloc(prefixlen + 1);
+        result = checked_malloc(prefixlen + 1);
         memcpy(result, path, prefixlen);
         result[prefixlen] = 0;
         return result;
@@ -385,19 +391,7 @@ static int is_memsize_arg(char *argv[], int argi, int argc, int *merge_core_page
         if ((argi+1) >= argc) lose("missing argument for --dynamic-space-size");
         dynamic_space_size = parse_size_arg(argv[argi+1],
                                             "--dynamic-space-size");
-#ifdef MAX_DYNAMIC_SPACE_END
-        if (!((DYNAMIC_SPACE_START <
-                       DYNAMIC_SPACE_START+dynamic_space_size) &&
-                      (DYNAMIC_SPACE_START+dynamic_space_size <=
-                       MAX_DYNAMIC_SPACE_END))) {
-            char* suffix = "";
-            char* size = argv[argi-1];
-            if (!strchr(size, 'B') && !strchr(size, 'b')) suffix = " [MB]";
-            lose("--dynamic-space-size argument %s%s is too large, max %lu KB",
-                 size, suffix, (MAX_DYNAMIC_SPACE_END-DYNAMIC_SPACE_START) / 1024);
-        }
-#endif
-        return 2;
+        return 2; // return number of elements of argv[] consumed
     }
     if (!strcmp(arg, "--control-stack-size")) {
         if ((argi+1) >= argc) lose("missing argument for --control-stack-size");
@@ -466,13 +460,13 @@ parse_argv(struct memsize_options memsize_options,
         int output_index = 1;
 
 #ifndef LISP_FEATURE_WIN32
-        sbcl_argv = successful_malloc((argc + 1) * sizeof(char *));
+        sbcl_argv = checked_malloc((argc + 1) * sizeof(char *));
         char **argv_source = argv;
 #else
         int wargc;
         wchar_t **argv_source;
         argv_source = CommandLineToArgvW(GetCommandLineW(), &wargc);
-        sbcl_argv = successful_malloc((wargc + 1) * sizeof(wchar_t *));
+        sbcl_argv = checked_malloc((wargc + 1) * sizeof(wchar_t *));
 #endif
         sbcl_argv[0] = argv_source[0];
 
@@ -559,7 +553,7 @@ parse_argv(struct memsize_options memsize_options,
 #ifndef LISP_FEATURE_WIN32
             /* (argc - argi) for the arguments, one for the binary,
                and one for the terminating NULL. */
-            sbcl_argv = successful_malloc((2 + argc - argi) * sizeof(char *));
+            sbcl_argv = checked_malloc((2 + argc - argi) * sizeof(char *));
             sbcl_argv[0] = argv[0];
             while (argi < argc) {
                 char *arg = argv[argi++];
@@ -581,7 +575,7 @@ parse_argv(struct memsize_options memsize_options,
             int wargc;
             wchar_t** wargv;
             wargv = CommandLineToArgvW(GetCommandLineW(), &wargc);
-            sbcl_argv = successful_malloc((((argi < wargc) ? (wargc - argi) : 0) + 2)
+            sbcl_argv = checked_malloc((((argi < wargc) ? (wargc - argi) : 0) + 2)
                                           * sizeof(wchar_t *));
             sbcl_argv[0] = wargv[0];
             while (argi < wargc) {
@@ -641,6 +635,8 @@ initialize_lisp(int argc, char *argv[], char *envp[])
     lispobj initial_function;
     struct memsize_options memsize_options;
     memsize_options.present_in_core = 0;
+    extern void sb_query_os_page_size();
+    sb_query_os_page_size();
 
     bool have_hardwired_spaces = os_preinit(argv, envp);
 
@@ -701,7 +697,7 @@ initialize_lisp(int argc, char *argv[], char *envp[])
     // FIXME: if the 'have' flag is 0 and you've disabled disabling of ASLR
     // then we haven't done an exec(), nor unmapped the mappings that were obtained
     // already obtained (if any) so it is unhelpful to try again here.
-    allocate_lisp_dynamic_space(have_hardwired_spaces);
+    if (!have_hardwired_spaces) allocate_hardwired_spaces(1);
     gc_init();
 
     /* If no core file was specified, look for one. */
@@ -709,7 +705,8 @@ initialize_lisp(int argc, char *argv[], char *envp[])
     if (!core && !(core = search_for_core())) {
       /* Try resolving symlinks */
       if (sbcl_runtime) {
-        free(sbcl_runtime_home);
+        if (sbcl_runtime_home != libpath)
+            free(sbcl_runtime_home);
         char* real = sb_realpath(sbcl_runtime);
         if (!real)
           goto lose;
@@ -768,6 +765,7 @@ initialize_lisp(int argc, char *argv[], char *envp[])
     if (!options.disable_lossage_handler_p)
         enable_lossage_handler();
 
+    ensure_undefined_alien();
     os_link_runtime();
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
 #ifdef CALLBACK_WRAPPER_TRAMPOLINE // not defined if #-sb-thread
@@ -810,3 +808,5 @@ initialize_lisp(int argc, char *argv[], char *envp[])
     create_main_lisp_thread(initial_function);
     return 0;
 }
+
+int lisp_gc_strategy_id() { return GC_STRATEGY_ID; }
