@@ -1,4 +1,5 @@
-#+(or (not system-tlabs) interpreter) (invoke-restart 'run-tests::skip-file)
+#+(or gc-stress ;; c-find-heap->arena is not gc-safe
+      (not system-tlabs) interpreter) (invoke-restart 'run-tests::skip-file)
 
 (in-package sb-vm)
 
@@ -580,6 +581,19 @@
 (defun g (n) (make-array (the integer n) :initial-element #\z))
 (defun f (a n) (with-arena (a) (g n)))
 
+(defun test-read-line ()
+  (let ((stream
+         (make-string-input-stream (format nil "this is line1 and it should be long enough to ~
+ require doubling at least once in ansi-stream-read-line so ok then ~
+ here are some more characters for you because why not~%line2"))))
+    (let* ((a (sb-vm::new-arena 1048576))
+           (string (sb-vm:with-arena (a) (read-line stream))))
+      string)))
+
+(test-util:with-test (:name :read-line)
+  (test-read-line)
+  (assert (null (c-find-heap->arena))))
+
 (defvar *vect* (f *another-arena* 10))
 (setf (aref *vect* 3) "foo")
 
