@@ -646,3 +646,36 @@
     ((most-negative-fixnum) nil)
     (((1+ most-negative-fixnum)) nil)
     (((1- most-positive-fixnum)) nil)))
+
+(with-test (:name :logior-derive-negative)
+  (assert-type
+   (lambda (x)
+     (declare (fixnum x))
+     (logior x 1))
+   (or (integer #.(1+ most-negative-fixnum) -1)
+       (integer 1 #.most-positive-fixnum)))
+  (assert-type
+   (lambda (x y)
+     (declare (fixnum x)
+              ((and (signed-byte 16) (not (eql 0))) y))
+     (logior y x))
+   (or (integer #.(1+ most-negative-fixnum) -1)
+       (integer 1 #.most-positive-fixnum)))
+  (assert-type
+   (lambda (x)
+     (declare (fixnum x))
+     (logior x 10))
+   (or (integer #.(+ most-negative-fixnum 10) -1)
+       (integer 10 #.most-positive-fixnum))))
+
+(with-test (:name :range<=low-address)
+  (checked-compile-and-assert
+   ()
+   `(lambda (a)
+      (typep a '(integer 1 13)))
+    ((0.0) nil)
+    ((0) nil)
+    ((1) t)
+    ((6) t)
+    ((13) t)
+    ((14) nil)))
