@@ -1304,22 +1304,21 @@
                    (read-line f)))))
 
 ;; test for lp#659107
-(with-test (:name :cmdline-setq-external-format
-                  :skipped-on (not :sb-unicode))
+(with-test (:name :cmdline-setq-external-format)
   (with-scratch-file (script "lisp")
     (with-open-file (stream script :direction :output
                                    :if-exists :supersede
                                    :if-does-not-exist :create
-                                   :external-format :utf16le)
-      (format stream "(defvar s \"what? ~A\"~%)" (name-char "GRINNING_FACE"))
+                                   :external-format :latin1)
+      (format stream "(defvar s \"what? ~A\"~%)" (name-char "LATIN_SMALL_LETTER_Y_WITH_DIAERESIS"))
       (format stream "(sb-ext:exit :code
- (if (and (string= (subseq s 0 6) \"what? \") (char= (char s 6) #\\grinning_face)) 0 1))~%"))
+ (if (and (string= (subseq s 0 6) \"what? \") (char= (char s 6) #\\LATIN_SMALL_LETTER_Y_WITH_DIAERESIS)) 0 1))~%"))
     (let ((process (run-program
                     sb-ext:*runtime-pathname*
                     (list "--core" sb-int:*core-string*
                           "--noinform" "--no-sysinit" "--no-userinit" "--noprint"
                           "--disable-debugger"
-                          "--eval" "(setq *default-external-format* :utf16le)"
+                          "--eval" "(setq *default-external-format* :latin1)"
                           "--load" script)
                     :error t)))
         (assert (zerop (process-exit-code process))))))
@@ -1349,7 +1348,8 @@
           (*error-output* (make-broadcast-stream)))
       (multiple-value-bind (output failurep warningsp)
           (compile-file path :verbose nil)
-        (declare (ignore output))
+        (delete-file output)
+        (delete-file path)
         (assert (null failurep))
         (assert (null warningsp))))))
 
