@@ -1,5 +1,5 @@
 ;;;; This file is for testing external-format functionality for
-;;;; ISO-8859-2, using test machinery which does not have side
+;;;; ISO-8859-1, using test machinery which does not have side
 ;;;; effects.  Note that the tests here reach into unexported
 ;;;; functionality, and should not be used as a guide for users.
 
@@ -14,8 +14,6 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
-#+(or (not sb-unicode) unicode-lite) (invoke-restart 'run-tests::skip-file)
-
 (defvar *test-path* (scratch-file-name))
 
 (macrolet ((input-test (inxf expected &environment env)
@@ -61,37 +59,38 @@
                   (macrolet ((test (outxf expected)
                                `(output-test chars ,outxf ,expected)))
                     ,@body)))))
-  (with-input-bytes ((:input :lf) '(#xb7 #x0a #x37))
-    (test :iso-8859-2 '(#\CARON #\Newline #\7))
-    (test (:iso-8859-2 :newline :lf) '(#\CARON #\Newline #\7))
-    (test (:iso-8859-2 :newline :cr) '(#\CARON #\Newline #\7))
-    (test (:iso-8859-2 :newline :crlf) '(#\CARON #\Newline #\7)))
-  (with-output-characters ((:output :lf) '(#\CARON #\Linefeed #\7))
-    (test :iso-8859-2 '(#xb7 #x0a #x37))
-    (test (:iso-8859-2 :newline :lf) '(#xb7 #x0a #x37))
-    (test (:iso-8859-2 :newline :cr) '(#xb7 #x0d #x37))
-    (test (:iso-8859-2 :newline :crlf) '(#xb7 #x0d #x0a #x37)))
-  (with-input-bytes ((:input :cr) '(#xb7 #x0d #x37))
-    (test :iso-8859-2 '(#\CARON #\Return #\7))
-    (test (:iso-8859-2 :newline :lf) '(#\CARON #\Return #\7))
-    (test (:iso-8859-2 :newline :cr) '(#\CARON #\Newline #\7))
-    (test (:iso-8859-2 :newline :crlf) '(#\CARON #\Return #\7)))
-  (with-output-characters ((:output :cr) '(#\CARON #\Return #\7))
-    (test :iso-8859-2 '(#xb7 #x0d #x37))
-    (test (:iso-8859-2 :newline :lf) '(#xb7 #x0d #x37))
-    (test (:iso-8859-2 :newline :cr) '(#xb7 #x0d #x37))
-    (test (:iso-8859-2 :newline :crlf) '(#xb7 #x0d #x37)))
-  (with-input-bytes ((:input :crlf) '(#xb7 #x0d #x0a #x37))
-    (test :iso-8859-2 '(#\CARON #\Return #\Newline #\7))
-    (test (:iso-8859-2 :newline :lf) '(#\CARON #\Return #\Newline #\7))
-    (test (:iso-8859-2 :newline :cr) '(#\CARON #\Newline #\Newline #\7))
-    (test (:iso-8859-2 :newline :crlf) '(#\CARON #\Newline #\7)))
-  (with-output-characters ((:output :crlf) '(#\CARON #\Return #\Linefeed #\7))
-    (test :iso-8859-2 '(#xb7 #x0d #x0a #x37))
-    (test (:iso-8859-2 :newline :lf) '(#xb7 #x0d #x0a #x37))
-    (test (:iso-8859-2 :newline :cr) '(#xb7 #x0d #x0d #x37))
-    (test (:iso-8859-2 :newline :crlf) '(#xb7 #x0d #x0d #x0a #x37))))
+  (with-input-bytes ((:input :lf) '(#x35 #x0a #x37))
+    (test :iso-8859-1 '(#\5 #\Newline #\7))
+    (test (:iso-8859-1 :newline :lf) '(#\5 #\Newline #\7))
+    (test (:iso-8859-1 :newline :cr) '(#\5 #\Newline #\7))
+    (test (:iso-8859-1 :newline :crlf) '(#\5 #\Newline #\7)))
+  (with-output-characters ((:output :lf) '(#\5 #\Linefeed #\7))
+    (test :iso-8859-1 '(#x35 #x0a #x37))
+    (test (:iso-8859-1 :newline :lf) '(#x35 #x0a #x37))
+    (test (:iso-8859-1 :newline :cr) '(#x35 #x0d #x37))
+    (test (:iso-8859-1 :newline :crlf) '(#x35 #x0d #x0a #x37)))
+  (with-input-bytes ((:input :cr) '(#x35 #x0d #x37))
+    (test :iso-8859-1 '(#\5 #\Return #\7))
+    (test (:iso-8859-1 :newline :lf) '(#\5 #\Return #\7))
+    (test (:iso-8859-1 :newline :cr) '(#\5 #\Newline #\7))
+    (test (:iso-8859-1 :newline :crlf) '(#\5 #\Return #\7)))
+  (with-output-characters ((:output :cr) '(#\5 #\Return #\7))
+    (test :iso-8859-1 '(#x35 #x0d #x37))
+    (test (:iso-8859-1 :newline :lf) '(#x35 #x0d #x37))
+    (test (:iso-8859-1 :newline :cr) '(#x35 #x0d #x37))
+    (test (:iso-8859-1 :newline :crlf) '(#x35 #x0d #x37)))
+  (with-input-bytes ((:input :crlf) '(#x35 #x0d #x0a #x37))
+    (test :iso-8859-1 '(#\5 #\Return #\Newline #\7))
+    (test (:iso-8859-1 :newline :lf) '(#\5 #\Return #\Newline #\7))
+    (test (:iso-8859-1 :newline :cr) '(#\5 #\Newline #\Newline #\7))
+    (test (:iso-8859-1 :newline :crlf) '(#\5 #\Newline #\7)))
+  (with-output-characters ((:output :crlf) '(#\5 #\Return #\Linefeed #\7))
+    (test :iso-8859-1 '(#x35 #x0d #x0a #x37))
+    (test (:iso-8859-1 :newline :lf) '(#x35 #x0d #x0a #x37))
+    (test (:iso-8859-1 :newline :cr) '(#x35 #x0d #x0d #x37))
+    (test (:iso-8859-1 :newline :crlf) '(#x35 #x0d #x0d #x0a #x37))))
 
+#+sb-unicode
 (macrolet ((output-test (chars outxf expected &environment env)
              `(progn
                 (with-open-file (s *test-path* :element-type 'character
@@ -118,20 +117,16 @@
                   (macrolet ((test (outxf expected)
                                `(output-test chars ,outxf ,expected)))
                     ,@body)))))
-  (with-output-characters ((:output :invalid :lf) (list #\CARON (code-char 512) #\Linefeed #\7))
-    (test :iso-8859-2 '(#xb7 #x0a #x37))
-    (test (:iso-8859-2 :replacement #\?) '(#xb7 #x3f #x0a #x37))
-    (test (:iso-8859-2 :newline :lf) '(#xb7 #x0a #x37))
-    (test (:iso-8859-2 :newline :lf :replacement #\?) '(#xb7 #x3f #x0a #x37))
-    (test (:iso-8859-2 :newline :cr) '(#xb7 #x0d #x37))
-    (test (:iso-8859-2 :newline :cr :replacement #\?) '(#xb7 #x3f #x0d #x37))
-    (test (:iso-8859-2 :newline :crlf) '(#xb7 #x0d #x0a #x37))
-    (test (:iso-8859-2 :newline :crlf :replacement #\?) '(#xb7 #x3f #x0d #x0a #x37))))
+  (with-output-characters ((:output :invalid :lf) (list #\5 (code-char 512) #\Linefeed #\7))
+    (test :iso-8859-1 '(#x35 #x0a #x37))
+    (test (:iso-8859-1 :newline :lf) '(#x35 #x0a #x37))
+    (test (:iso-8859-1 :newline :cr) '(#x35 #x0d #x37))
+    (test (:iso-8859-1 :newline :crlf) '(#x35 #x0d #x0a #x37))))
 
 (macrolet ((test (inxf expected &environment env)
              `(with-test (:name (,(macroexpand 'name env) ,inxf))
                 (with-open-file (s *test-path* :external-format ',inxf)
-                  (let* ((string (make-string 10000))
+                  (let* ((string (make-string 100000))
                          (count (read-sequence string s)))
                     (assert (equal (map 'list 'char-code (subseq string 0 count)) ,expected))))))
            (with-test-file ((id bytes) &body body)
@@ -145,20 +140,20 @@
            (tests (size)
              `(progn
                 (with-test-file ((:input :lf ,size) (contents ,size '(#x0a)))
-                  (test :iso-8859-2 (contents ,size '(10)))
-                  (test (:iso-8859-2 :newline :lf) (contents ,size '(10)))
-                  (test (:iso-8859-2 :newline :cr) (contents ,size '(10)))
-                  (test (:iso-8859-2 :newline :crlf) (contents ,size '(10))))
+                  (test :iso-8859-1 (contents ,size '(10)))
+                  (test (:iso-8859-1 :newline :lf) (contents ,size '(10)))
+                  (test (:iso-8859-1 :newline :cr) (contents ,size '(10)))
+                  (test (:iso-8859-1 :newline :crlf) (contents ,size '(10))))
                 (with-test-file ((:input :cr ,size) (contents ,size '(#x0d)))
-                  (test :iso-8859-2 (contents ,size '(13)))
-                  (test (:iso-8859-2 :newline :lf) (contents ,size '(13)))
-                  (test (:iso-8859-2 :newline :cr) (contents ,size '(10)))
-                  (test (:iso-8859-2 :newline :crlf) (contents ,size '(13))))
+                  (test :iso-8859-1 (contents ,size '(13)))
+                  (test (:iso-8859-1 :newline :lf) (contents ,size '(13)))
+                  (test (:iso-8859-1 :newline :cr) (contents ,size '(10)))
+                  (test (:iso-8859-1 :newline :crlf) (contents ,size '(13))))
                 (with-test-file ((:input :crlf ,size) (contents ,size '(#x0d #x0a)))
-                  (test :iso-8859-2 (contents ,size '(13 10)))
-                  (test (:iso-8859-2 :newline :lf) (contents ,size '(13 10)))
-                  (test (:iso-8859-2 :newline :cr) (contents ,size '(10 10)))
-                  (test (:iso-8859-2 :newline :crlf) (contents ,(1- size) '(10)))))))
+                  (test :iso-8859-1 (contents ,size '(13 10)))
+                  (test (:iso-8859-1 :newline :lf) (contents ,size '(13 10)))
+                  (test (:iso-8859-1 :newline :cr) (contents ,size '(10 10)))
+                  (test (:iso-8859-1 :newline :crlf) (contents ,(1- size) '(10)))))))
   (flet ((contents (size nl)
            (let ((bytes (make-array size :initial-element #x61)))
              (loop for x in nl
@@ -181,14 +176,11 @@
     (tests 516)
     (tests 517)
 
-    (with-test (:name :fd-stream-bytes-per-buffer)
-      (assert (= sb-impl::+bytes-per-buffer+ 8192)))
-
-    (tests 8190)
-    (tests 8191)
-    (tests 8192)
-    (tests 8193)
-    (tests 8194)))
+    (tests #.(- sb-impl::+bytes-per-buffer+ 2))
+    (tests #.(- sb-impl::+bytes-per-buffer+ 1))
+    (tests #.sb-impl::+bytes-per-buffer+)
+    (tests #.(+ sb-impl::+bytes-per-buffer+ 1))
+    (tests #.(+ sb-impl::+bytes-per-buffer+ 3))))
 
 (macrolet ((test (inxf expected &environment env)
              `(progn
@@ -229,21 +221,55 @@
                     (write-byte byte s)))
                 (symbol-macrolet ((name ,id))
                   ,@body))))
-  (with-test-file ((file-position :lf) '(#xb7 #x0a #x37 #x38 #x0a #x39 #x3a #x0a #x3b))
-    (test :iso-8859-2 (loop for i from 0 to 9 collect i))
-    (test (:iso-8859-2 :newline :lf) (loop for i from 0 to 9 collect i))
-    (test (:iso-8859-2 :newline :cr) (loop for i from 0 to 9 collect i))
-    (test (:iso-8859-2 :newline :crlf) (loop for i from 0 to 9 collect i)))
-  (with-test-file ((file-position :cr) '(#xb7 #x0d #x37 #x38 #x0d #x39 #x3a #x0d #x3b))
-    (test :iso-8859-2 (loop for i from 0 to 9 collect i))
-    (test (:iso-8859-2 :newline :lf) (loop for i from 0 to 9 collect i))
-    (test (:iso-8859-2 :newline :cr) (loop for i from 0 to 9 collect i))
-    (test (:iso-8859-2 :newline :crlf) (loop for i from 0 to 9 collect i)))
-  (with-test-file ((file-position :crlf) '(#xb7 #x0d #x0a #x37 #x38 #x0d #x0a #x39 #x3a #x0d #x0a #x3b))
-    (test :iso-8859-2 (loop for i from 0 to 12 collect i))
-    (test (:iso-8859-2 :newline :lf) (loop for i from 0 to 12 collect i))
-    (test (:iso-8859-2 :newline :cr) (loop for i from 0 to 12 collect i))
-    (test (:iso-8859-2 :newline :crlf) '(0 1 3 4 5 7 8 9 11 12))))
+  (with-test-file ((file-position :lf) '(#x35 #x0a #x37 #x38 #x0a #x39 #x3a #x0a #x3b))
+    (test :iso-8859-1 (loop for i from 0 to 9 collect i))
+    (test (:iso-8859-1 :newline :lf) (loop for i from 0 to 9 collect i))
+    (test (:iso-8859-1 :newline :cr) (loop for i from 0 to 9 collect i))
+    (test (:iso-8859-1 :newline :crlf) (loop for i from 0 to 9 collect i)))
+  (with-test-file ((file-position :cr) '(#x35 #x0d #x37 #x38 #x0d #x39 #x3a #x0d #x3b))
+    (test :iso-8859-1 (loop for i from 0 to 9 collect i))
+    (test (:iso-8859-1 :newline :lf) (loop for i from 0 to 9 collect i))
+    (test (:iso-8859-1 :newline :cr) (loop for i from 0 to 9 collect i))
+    (test (:iso-8859-1 :newline :crlf) (loop for i from 0 to 9 collect i)))
+  (with-test-file ((file-position :crlf) '(#x35 #x0d #x0a #x37 #x38 #x0d #x0a #x39 #x3a #x0d #x0a #x3b))
+    (test :iso-8859-1 (loop for i from 0 to 12 collect i))
+    (test (:iso-8859-1 :newline :lf) (loop for i from 0 to 12 collect i))
+    (test (:iso-8859-1 :newline :cr) (loop for i from 0 to 12 collect i))
+    (test (:iso-8859-1 :newline :crlf) '(0 1 3 4 5 7 8 9 11 12))))
+
+(macrolet ((output-test (chars outxf expected &environment env)
+             `(progn
+                (with-test (:name (,(macroexpand 'name env) write-string string ,outxf))
+                  (with-open-file (s *test-path* :element-type 'character
+                                     :external-format ',outxf
+                                     :direction :output :if-exists :supersede)
+                    (let ((string (coerce ,chars 'string)))
+                      (write-string string s)))
+                  (with-open-file (s *test-path* :element-type '(unsigned-byte 8))
+                    (let* ((vector (make-array 20 :element-type '(unsigned-byte 8)))
+                           (count (read-sequence vector s)))
+                      (assert (equal (map 'list 'identity (subseq vector 0 count)) ,expected)))))
+                (with-test (:name (,(macroexpand 'name env) write-string base-string ,outxf))
+                  (with-open-file (s *test-path* :element-type 'character
+                                     :external-format ',outxf
+                                     :direction :output :if-exists :supersede)
+                    (let ((string (coerce ,chars 'base-string)))
+                      (write-string string s)))
+                  (with-open-file (s *test-path* :element-type '(unsigned-byte 8))
+                    (let* ((vector (make-array 20 :element-type '(unsigned-byte 8)))
+                           (count (read-sequence vector s)))
+                      (assert (equal (map 'list 'identity (subseq vector 0 count)) ,expected)))))))
+           (with-output-characters ((id chars) &body body)
+             `(let ((chars ,chars))
+                (symbol-macrolet ((name ,id))
+                  (macrolet ((test (outxf expected)
+                               `(output-test chars ,outxf ,expected)))
+                    ,@body)))))
+  (with-output-characters ((:output :lf) '(#\5 #\Newline #\7))
+    (test :iso-8859-1 '(#x35 #x0a #x37))
+    (test (:iso-8859-1 :newline :lf) '(#x35 #x0a #x37))
+    (test (:iso-8859-1 :newline :cr) '(#x35 #x0d #x37))
+    (test (:iso-8859-1 :newline :crlf) '(#x35 #x0d #x0a #x37))))
 
 (macrolet ((output-test (chars outxf expected &environment env)
              `(progn
@@ -276,20 +302,21 @@
                                `(output-test chars ,outxf ,expected)))
                     ,@body)))))
   (with-output-characters ((:output :lf) (list #\5 #\Linefeed #\7))
-    (test :iso-8859-2 '(1 1 1))
-    (test (:iso-8859-2 :newline :lf) '(1 1 1))
-    (test (:iso-8859-2 :newline :cr) '(1 1 1))
-    (test (:iso-8859-2 :newline :crlf) '(1 2 1)))
-  (with-output-characters ((:output :invalid :lf) (list #\5 (code-char #xa1) #\Linefeed #\7))
+    (test :iso-8859-1 '(1 1 1))
+    (test (:iso-8859-1 :newline :lf) '(1 1 1))
+    (test (:iso-8859-1 :newline :cr) '(1 1 1))
+    (test (:iso-8859-1 :newline :crlf) '(1 2 1)))
+  #+sb-unicode
+  (with-output-characters ((:output :invalid :lf) (list #\5 (code-char 512) #\Linefeed #\7))
     ;; A sufficiently-smart streams implementation could statically determine the lengths
     ;; of replacement characters given as part of the external format
-    (test :iso-8859-2 '(1 nil 1 1))
-    (test (:iso-8859-2 :replacement #\?) '(1 nil 1 1))
-    (test (:iso-8859-2 :newline :lf) '(1 nil 1 1))
-    (test (:iso-8859-2 :newline :lf :replacement #\?) '(1 nil 1 1))
-    (test (:iso-8859-2 :newline :cr) '(1 nil 1 1))
-    (test (:iso-8859-2 :newline :cr :replacement #\?) '(1 nil 1 1))
-    (test (:iso-8859-2 :newline :crlf) '(1 nil 2 1))
-    (test (:iso-8859-2 :newline :crlf :replacement #\?) '(1 nil 2 1))))
+    (test :iso-8859-1 '(1 nil 1 1))
+    (test (:iso-8859-1 :replacement #\?) '(1 nil 1 1))
+    (test (:iso-8859-1 :newline :lf) '(1 nil 1 1))
+    (test (:iso-8859-1 :newline :lf :replacement #\?) '(1 nil 1 1))
+    (test (:iso-8859-1 :newline :cr) '(1 nil 1 1))
+    (test (:iso-8859-1 :newline :cr :replacement #\?) '(1 nil 1 1))
+    (test (:iso-8859-1 :newline :crlf) '(1 nil 2 1))
+    (test (:iso-8859-1 :newline :crlf :replacement #\?) '(1 nil 2 1))))
 
 (delete-file *test-path*)
