@@ -238,10 +238,7 @@
                (0 (= speed 0))
                (t t)))
            (extra-safe (&key speed safety &allow-other-keys)
-             (case safety
-               (0 (= speed 0))
-               (1 (< speed 2))
-               (t t)))
+             (>= safety 2))
            (test (type expr &key (filter #'safe))
              (checked-compile-and-assert
                  (:optimize `(:compilation-speed nil :space nil :filter ,filter)
@@ -256,6 +253,10 @@
           '(find :foo '(1 2 3 :foo) :start 3 :end 0 :from-end t))
     (test 'sb-kernel:bounding-indices-bad-error
           '(position :foo '(1 2 3 :foo) :start 3 :end 0 :from-end t))
+    (test 'sb-kernel:bounding-indices-bad-error
+          '(find :foo '(1 2 3) :start 5))
+    (test 'sb-kernel:bounding-indices-bad-error
+          '(position :foo '(1 2 3) :start 5 :from-end t))
     (test 'type-error
           '(let ((list (list 1 2 3 :foo)))
              (find :bar (nconc list list)))
@@ -497,7 +498,7 @@
     ;; Since there's no canonical order within a bin - we don't know
     ;; whether bin 0 is {:AND,AND} or {AND,:AND} - this gets tricky to check.
     ;; This is unfortunately a change-detector (if we alter SXHASH, or anything).
-    (assert (equalp (car consts) #(:and and :not not :or or 0 0)))))
+    (assert (equalp (car consts) #(:and and :or or :not not 0 0)))))
 
 (with-test (:name :memq-empty-seq)
   (assert (not (funcall (checked-compile '(lambda (x) (member x '()))) 1)))
@@ -678,7 +679,9 @@
     (check (find-if #'evenp y) (or integer null))
     (check (find-if #'evenp (the list y) :key #'car) list)
     (check (find x (the (simple-array character (*)) y)) (or character null))
-    (check (find x (the string y)) (or character null))))
+    (check (find x (the string y)) (or character null))
+    (check (find #\A y :test #'char=) (or (eql #\A) null))
+    (check (find #\a y :test #'char-equal) (or standard-char null))))
 
 (with-test (:name :position-type-derive)
   (macrolet
@@ -695,7 +698,9 @@
     (check (position x (the (simple-string 10) y)) (or (mod 10) null))
     (check (position x y :end 10) (or (mod 10) null))
     (check (position x (the cons y) :start 5 :end 10) (or (integer 5 9) null))
-    (check (position-if x y :end 10) (or (mod 10) null))))
+    (check (position-if x y :end 10) (or (mod 10) null))
+    (check (position x y :start (the (integer 10 20) x) :end x) (or (or (integer 10 19) null) null))
+    (check (position x (the (or null (simple-vector 5)) y)) (or null (mod 5)))))
 
 (with-test (:name :string-cmp)
   (macrolet
@@ -873,3 +878,84 @@
   (checked-compile `(lambda ()
                       (replace nil #()))
                    :allow-notes nil))
+
+(with-test (:name :reduce-type)
+  (assert-type
+   (lambda (v)
+     (declare ((vector (unsigned-byte 8)) v))
+     (reduce #'logior v))
+   (unsigned-byte 8)))
+
+(with-test (:name :position-index-errors)
+  (assert (= (count 'sb-int:sequence-bounding-indices-bad-error
+                    (ctu:ir1-named-calls `(lambda (y)
+                                            (declare (optimize speed)
+                                                     (simple-string y))
+                                            (position #\a y :start 1))))
+             1))
+  (assert (= (count 'sb-int:sequence-bounding-indices-bad-error
+                    (ctu:ir1-named-calls `(lambda (y)
+                                            (declare (optimize speed)
+                                                     (string y))
+                                            (find #\a y :start 1))))
+             2)))
+
+(with-test (:name :find-equalp-type)
+  (assert-type
+   (lambda (y j)
+     (declare ((or integer simple-string) j))
+     (find j y :test #'equalp))
+   (or array number null)))
+
+(with-test (:name :sequence-union-types)
+  (assert-type
+   (lambda (x)
+     (declare ((or cons (simple-vector 10)) x))
+     (copy-seq x))
+   (or cons (simple-vector 10)))
+  (assert-type
+   (lambda (x)
+     (declare ((or list (vector t 10)) x))
+     (reverse x))
+   (or list simple-vector))
+  (assert-type
+   (lambda (x)
+     (declare (cons x))
+     (copy-seq x))
+   cons)
+  (assert-type
+   (lambda (x)
+     (declare ((simple-vector 10) x))
+     (remove 10 x))
+   simple-vector))
+
+(with-test (:name :concatenate-list-type)
+  (assert-type
+   (lambda (x)
+     (concatenate 'list '(1 2 3) x))
+   cons)
+  (assert-type
+   (lambda (x)
+     (declare (cons x))
+     (concatenate 'list x x))
+   cons)
+  (assert-type
+   (lambda (x)
+     (declare ((or cons (simple-vector 10)) x))
+     (concatenate 'list x x))
+   cons))
+
+(with-test (:name :sequencep-test)
+  (assert (not (ctu:ir1-named-calls `(lambda (a)
+                                       (declare ((array t) a))
+                                       (the sequence a))))))
+
+(with-test (:name :find-test-not-type)
+  (assert-type
+   (lambda (s j)
+     (find j s :key #'car :test-not #'char=))
+   list)
+  (assert-type
+   (lambda (s j)
+     (find j s :test-not #'char=))
+   (or character null)))
