@@ -85,13 +85,13 @@
                                (in-bounds-p target-addr (core-linkage-bounds core)))
                        (push (list (dstate-cur-offs dstate)
                                    4    ; length
-                                   "adrp-gotpcrel"
+                                   :adrp-gotpcrel
                                    target-addr
                                    (format nil "x~d" reg))
                              list)
                        (push (list (+ 4 (dstate-cur-offs dstate))
                                    4    ; length
-                                   "ldr-gotpcrel"
+                                   :ldr-gotpcrel
                                    target-addr
                                    (format nil "x~d" reg))
                              list))))
@@ -108,13 +108,13 @@
                                (in-bounds-p target-addr (core-linkage-bounds core)))
                        (push (list (dstate-cur-offs dstate)
                                    4    ; length
-                                   "adrp-gotpcrel"
+                                   :adrp-gotpcrel
                                    target-addr
                                    (format nil "x~d" reg))
                              list)
                        (push (list (+ 4 (dstate-cur-offs dstate))
                                    4    ; length
-                                   "ldr-gotpcrel"
+                                   :ldr-gotpcrel
                                    target-addr
                                    (format nil "x~d" reg))
                              list)))))))
@@ -128,7 +128,7 @@
                       (in-bounds-p target-addr (core-linkage-bounds core)))
               (push (list* (dstate-cur-offs dstate)
                            4            ; length
-                           (if (eq inst inst-bl) "bl" "b")
+                           (if (eq inst inst-bl) :bl :b)
                            target-addr)
                     list))))))
      seg
@@ -157,7 +157,7 @@
              (machine-ea-disp x)))
       (when (and emit-cfi
                  (equalp (car insts) '(0 pop (#s(machine-ea :disp 8 :base 5) . :qword))))
-        (push (list* 0 3 "pop" "8(%rbp)") result))
+        (push (list* 0 3 :pop "8(%rbp)") result))
       (do ((insts insts (cdr insts)))
           ((endp insts))
         (let* ((inst (car insts))
@@ -169,9 +169,7 @@
                    (aver (eq (cadr inst) 'sb-x86-64-asm::call))
                    (aver (< ea alien-linkage-end)) ; CALL via alien linkage
                    (aver (= inst-len 5))
-                   ;; Note: This defers to the ASM writer inference of which alien symbol
-                   ;; is referenced which in retrospect is a bad approach.
-                   (push (list* (car inst) 5 "call" ea) result))
+                   (push (list* (car inst) 5 :call ea) result))
                   ((pc-relative-ea-p ea)
                    (let* ((next-pc (+ load-addr (caadr insts)))
                           (ea (+ next-pc (ea-disp-of ea)))
@@ -180,16 +178,15 @@
                        (cond ((< ea alien-linkage-end) ; alien linkage
                               (aver (= inst-len 7))
                               (let ((op (ecase (cadr inst)
-                                          (sb-x86-64-asm::mov "mov-gotpcrel")
-                                          (sb-x86-64-asm::lea "lea")))
+                                          (sb-x86-64-asm::mov :mov-gotpcrel)
+                                          (sb-x86-64-asm::lea :lea)))
                                     (args (list ea (string-downcase (princ-to-string (third inst))))))
-                                ;; same Note as above pertains here
                                 (push (list* (car inst) 7 op args) result)))
                              ((eq (cadr inst) 'sb-x86-64-asm::lea)
                               ;; Get ADDRESS of lisp linkage cell in stepping-enabled code
                               (aver (eq (third inst) (get-gpr :qword 0))) ; %rax
                               (aver (= inst-len 7))
-                              (push (list* (car inst) 7 "lea" (format nil "(fntbl+~d)(%rip),%rax"
+                              (push (list* (car inst) 7 :lea (format nil "(fntbl+~d)(%rip),%rax"
                                                                       table-offset))
                                     result))
                              (t ; lisp CALL or JMP
@@ -198,9 +195,18 @@
                                      (format nil "~a *(fntbl+~d)(%rip)"
                                              (string-downcase (cadr inst))
                                              table-offset)))
-                                (push (list* (car inst) 6 "lispcall" new-inst) result))))))))))))
+                                (push (list* (car inst) 6 :lispcall new-inst) result))))))))))))
       (nreverse result)))
 
+(defun c-linkage-sym-from-addr (addr core)
+  (let ((entry-index (/ (- addr (bounds-low (core-linkage-bounds core)))
+                        (core-alien-linkage-entry-size core))))
+    (setf (bit (core-alien-linkage-symbol-usedp core) entry-index) 1)
+    (let ((symbol (aref (core-alien-linkage-symbols core) entry-index)))
+      (if (listp symbol)
+          (values (car symbol) entry-index t) ; data linkage
+          (values symbol entry-index nil))))) ; code linkage
+
 ;;; Using assembler directives and/or real mnemonics, dump COUNT bytes
 ;;; of memory at PADDR (physical addr) to STREAM.
 ;;; The function's address as per the core file is VADDR.
@@ -286,76 +292,47 @@
           ;; to see them where they belong in the instruction stream]
           (when (and instructions (= (caar instructions) cur-offset))
             (destructuring-bind (length opcode . operand) (cdr (pop instructions))
-              (when (cond ((member opcode #+arm64 '("bl" "b")
-                                          #+x86-64 '("jmp" "je" "call")
-                                          :test #'string=)
-                           (when (in-bounds-p operand (core-linkage-bounds core))
-                             (let ((entry-index
-                                     (/ (- operand (bounds-low (core-linkage-bounds core)))
-                                        (core-alien-linkage-entry-size core))))
-                               (setf (bit (core-alien-linkage-symbol-usedp core) entry-index) 1
-                                     operand (aref (core-alien-linkage-symbols core) entry-index))))
-                           (format stream " ~A ~:[0x~X~;~a~:[~;@PLT~]~]~%"
+              (ecase opcode
+                (#+arm64 (:bl :b) #+x86-64 (:call)
+                 ;; XXX Should this AVER that it is in-bounds?
+                 (when (in-bounds-p operand (core-linkage-bounds core))
+                   (setq operand (c-linkage-sym-from-addr operand core)))
+                 (format stream " ~A ~:[0x~X~;~a~:[~;@PLT~]~]~%"
                                    opcode (stringp operand) operand
                                    #+x86-64
                                    (core-enable-pie core)
                                    #+arm64 nil ; arm64 doesn't need the extra @PLT
                                    ))
-                          ((or #+x86-64 (string= opcode "mov-gotpcrel")
-                               #+arm64 (string= opcode "ldr-gotpcrel")
-                               #+arm64 (string= opcode "adrp-gotpcrel"))
-                           (let* ((entry-index
-                                    (/ (- (car operand) (bounds-low (core-linkage-bounds core)))
-                                       (core-alien-linkage-entry-size core)))
-                                  (c-symbol (let ((thing (aref (core-alien-linkage-symbols core)
-                                                               entry-index)))
-                                              (if (consp thing) (car thing) thing))))
-                             (setf (bit (core-alien-linkage-symbol-usedp core) entry-index) 1)
-                             #+x86-64
-                             (format stream " mov ~A@GOTPCREL(%rip), %~(~A~)~%" c-symbol (cadr operand))
-                             #+arm64
-                             (cond ((string= opcode "adrp-gotpcrel")
-                                    (format stream " adrp ~A,:got:~A~%" (cadr operand) c-symbol))
-                                   ((string= opcode "ldr-gotpcrel")
-                                    (format stream " ldr ~A, [~A, #:got_lo12:~A]~%"
-                                            (cadr operand)
-                                            (cadr operand)
-                                            c-symbol))
-                                   (t (error "unreachable")))))
-                          #+x86-64
-                          ((string= opcode "lea")
-                           (if (stringp operand)
-                               (format stream " lea ~A~%" operand) ; the "operand" is the whole instructon
-                               (let* ((entry-index
-                                       (/ (- (car operand) (bounds-low (core-linkage-bounds core)))
-                                          (core-alien-linkage-entry-size core)))
-                                      (c-symbol (aref (core-alien-linkage-symbols core) entry-index)))
-                                 (setf (bit (core-alien-linkage-symbol-usedp core) entry-index) 1)
-                                 ;; lea becomes "mov" with gotpcrel as src, which will become lea
-                                 (format stream " mov ~A@GOTPCREL(%rip), %~(~A~)~%" c-symbol (cadr operand)))))
-                          #+x86-64
-                          ((string= opcode "pop")
-                           (format stream " ~A ~A~%" opcode operand)
-                           (cond ((string= operand "8(%rbp)")
-                                  (format stream " .cfi_def_cfa 6, 16~% .cfi_offset 6, -16~%"))
-                                 ((string= operand "%rbp")
+                ((#+x86-64 :mov-gotpcrel) ; = get address of alien code linkage entry
+                 (let ((c-symbol (c-linkage-sym-from-addr (car operand) core)))
+                   (format stream " mov ~A@GOTPCREL(%rip), %~(~A~)~%" c-symbol (cadr operand))))
+                ((#+arm64 :adrp-gotpcrel)
+                 (let ((c-symbol (c-linkage-sym-from-addr (car operand) core)))
+                   (format stream " adrp ~A,:got:~A~%" (cadr operand) c-symbol)))
+                ((#+arm64 :ldr-gotpcrel)
+                 (let ((c-symbol (c-linkage-sym-from-addr (car operand) core)))
+                   (format stream " ldr ~A, [~A, #:got_lo12:~A]~%"
+                           (cadr operand) (cadr operand) c-symbol)))
+                ((#+x86-64 :lea)
+                 ;; The THEN case is to calculate a linkage-table cell address for stepping,
+                 ;; the ELSE case is FOREIGN-SYMBOL-SAP
+                 (if (stringp operand)
+                     (format stream " lea ~A~%" operand) ; the "operand" is the whole instructon
+                     (let ((c-symbol (c-linkage-sym-from-addr (car operand) core)))
+                       ;; lea becomes "mov" with gotpcrel as src, which will become lea
+                       (format stream " mov ~A@GOTPCREL(%rip), %~(~A~)~%" c-symbol (cadr operand)))))
+                ((#+x86-64 :pop)
+                 (format stream " ~A ~A~%" opcode operand)
+                 (cond ((string= operand "8(%rbp)")
+                        (format stream " .cfi_def_cfa 6, 16~% .cfi_offset 6, -16~%"))
+                       ((string= operand "%rbp")
                                         ;(format stream " .cfi_def_cfa 7, 8~%")
-                                  nil)
-                                 (t)))
-                          #+x86-64
-                          ((or (string= opcode "mov") (string= opcode "lispcall"))
+                        nil)
+                       (t)))
+                ((#+x86-64 :lispcall)
                            ;; the so-called "operand" is the entire instruction
                            (write-string operand stream)
-                           (terpri stream))
-                          ((or (string= opcode "call *") (string= opcode "jmp *"))
-                           ;; Indirect call - since the code is in immobile space,
-                           ;; we could render this as a 2-byte NOP followed by a direct
-                           ;; call. For simplicity I'm leaving it exactly as it was.
-                           (format stream " ~A(CS+0x~x)~%"
-                                   opcode ; contains a "*" as needed for the syntax
-                                   (- operand (bounds-low (core-code-bounds core)))))
-                          (t))
-                (bug "Random annotated opcode ~S" opcode))
+                           (terpri stream)))
               (incf ptr length)))
           (when (= cur-offset count) (return)))))
     (when emit-cfi
@@ -495,11 +472,14 @@
 (defconstant core-align #+x86-64 4096 #+arm64 65536)
 
 (defun write-preamble (output n-lisp-linkage-words)
-  #+linkage-space (format output "  .local fntbl
- .comm fntbl,~D,8
- .globl lisp_function_linkage_table
- .equiv lisp_function_linkage_table, fntbl
-" (* n-lisp-linkage-words n-word-bytes))
+  (declare (ignore n-lisp-linkage-words))
+  #+linkage-space (format output
+                          " .globl lisp_fun_linkage_space
+ .bss~% .align 8~% .size lisp_fun_linkage_space, ~D
+lisp_fun_linkage_space: .zero ~:*~D
+ .equiv fntbl, lisp_fun_linkage_space~%"
+              (ash 1 (+ sb-vm:n-linkage-index-bits sb-vm:word-shift)))
+
   (format output " .text~% .file \"sbcl.core\"
 ~:[~; .macro .size sym size # ignore
  .endm
@@ -693,7 +673,7 @@
                      (namestring
                       (sb-c::debug-source-namestring
                        (truly-the sb-c::debug-source (translate source spacemap)))))
-                (setq namestring (if (eq namestring (core-nil-object core))
+                (setq namestring (if (core-null-p namestring)
                                      "sbcl.core"
                                      (translate namestring spacemap)))
                 (unless (string= namestring prev-namestring)
@@ -913,7 +893,7 @@
                                       ; symbol table -- ^ ^ -- for which section
              (:note ".note.GNU-stack" ,+sht-progbits+ 0 0 0 1  0)))
          (extern-c-symbols
-          '("lisp_code_start" #+linkage-space "lisp_function_linkage_table"))
+          '("lisp_code_start" #+linkage-space "lisp_fun_linkage_space"))
          (string-table
           (string-table (append extern-c-symbols (map 'list #'second sections))))
          (strings (cdr string-table))
@@ -1164,6 +1144,10 @@
                  core-align))))
   fixups)
 
+(defun force-fntbl-ref-p (spacemap)
+  (let ((features (detect-target-features spacemap)))
+    (not (find :immobile-code features))))
+
 ;;; Given a native SBCL '.core' file, or one attached to the end of an executable,
 ;;; separate it into pieces.
 ;;; ASM-PATHNAME is the name of the assembler file that will hold all the Lisp code.
@@ -1194,6 +1178,10 @@
   ;; Remove old files
   (ignore-errors (delete-file asm-pathname))
   (ignore-errors (delete-file elf-core-pathname))
+  ;; Crappy assumption: we already called redirect-text-space-calls which
+  ;; assigned a value to *nil-taggedptr*. Anyway, why doesn't this use
+  ;; PARSE-CORE-HEADER like it should?
+  (assert (/= *nil-taggedptr* 0))
   ;; Ensure that all files can be opened
   (with-open-file (input input-pathname :element-type '(unsigned-byte 8))
     (with-open-file (asm-file asm-pathname :direction :output :if-exists :supersede)
@@ -1205,6 +1193,7 @@
           (case id
             (#.build-id-core-entry-type-code
              (when verbose
+               (incf ptr 2)
                (let ((string (make-string (%vector-raw-bits core-header ptr)
                                           :element-type 'base-char)))
                  (%byte-blt core-header (* (1+ ptr) n-word-bytes) string 0 (length string))
@@ -1315,7 +1304,7 @@
       (let ((where (ash (linkage-space-header-ptr linkage-space-info) word-shift)))
         ;; LINKAGE_SPACE core entry gets a linker fixup to the second ELF symbol
         (vector-push-extend `#(,R_ABS64 ,where 2 0) relocs)
-        (read-linkage-cells input linkage-space-info))
+        (read-linkage-cells input linkage-space-info core-offset))
       ;; Map the original core file to memory
       (with-mapped-core (sap core-offset original-total-npages input)
         (let* ((data-spaces
@@ -1398,7 +1387,11 @@
                (loop for s across (core-alien-linkage-symbols core)
                      do (format asm-file " .quad ~:[~;-1, ~]~a~%"
                                 (consp s)
-                                (if (consp s) (car s) s))))
+                                (if (consp s) (car s) s)))
+               ;; dark magic to ensure that the elf linkage space is emitted
+               (when (force-fntbl-ref-p spacemap)
+                 (format asm-file "# this is not an alien symbol~% .quad fntbl~%"))
+               )
               (t
                (format asm-file "~% .section .rodata~%")
                (format asm-file " .globl anchor_junk~%")
@@ -1418,7 +1411,7 @@
       (let* ((core-header (make-array +backend-page-bytes+
                                       :element-type '(unsigned-byte 8)))
              (core-offset (read-core-header input core-header nil))
-             (parsed-header (parse-core-header input core-header))
+             (parsed-header (parse-core-header input core-header core-offset))
              (space-list (core-header-space-list parsed-header)))
         ;; Map the core file to memory
         (with-mapped-core (sap core-offset (core-header-total-npages parsed-header) input)
@@ -1480,7 +1473,7 @@
 ;;; I don't feel like programmatically scanning the asm code to determine these.
 ;;; Hardcoded is good enough (until it isn't)
 (defparameter *c-linkage-redirects*
-  (mapcar (lambda (x) (cons x (foreign-symbol-sap x)))
+  (mapcar #'list
           '("switch_to_arena"
             "alloc"
             "alloc_list"
@@ -1508,8 +1501,17 @@
              (c-linkage-vector ; physical
               (%make-lisp-obj (logior (sap-int (sap+ (space-physaddr text-space spacemap)
                                                      code-offsets-vector-size))
-                                      other-pointer-lowtag))))
+                                      other-pointer-lowtag)))
+             (target-alien-lss ; linkage-space-start
+              (symbol-global-value
+               (find-target-symbol (package-id "SB-VM") "ALIEN-LINKAGE-SPACE-START"
+                                   spacemap))))
     (aver (<= (length items) (length c-linkage-vector)))
+    (dolist (x *c-linkage-redirects*)
+      (let* ((index (position (car x) (core-alien-linkage-symbols core)
+                              :test (lambda (a b) (and (stringp b) (string= a b)))))
+             (addr (+ target-alien-lss (* (core-alien-linkage-entry-size core) index))))
+        (rplacd x (int-sap addr))))
     (with-pinned-objects (inst-buffer)
       (do ((sap (vector-sap inst-buffer))
            (item-index 0 (1+ item-index))
@@ -1533,8 +1535,8 @@
         (when (eq (second inst) 'call)
           (let ((operand (third inst)))
             (when (and (integerp operand)
-                       (>= operand alien-linkage-space-start)
-                       (< operand (+ alien-linkage-space-start alien-linkage-space-size)))
+                       (>= operand target-alien-lss)
+                       (< operand (+ target-alien-lss alien-linkage-space-size)))
               (let* ((index (position (int-sap operand) *c-linkage-redirects*
                                       :key #'cdr :test #'sap=))
                      (branch-target (+ c-linkage-vector-vaddr
@@ -1617,11 +1619,13 @@
             (unless cell
               (setq cell (list linkage-index))
               (push cell indices))
-            ;; Collect list of insts replace for the particular linkage-index.
+            ;; Collect list of insts to replace for the particular linkage-index.
             (push inst (cdr cell)))))))
   ;; Change each linkage table call to instead go directly to the target
   ;; but only if the target uniquely identifies its linkage index within
   ;; this code component for purposes of undoing the optimization.
+  ;; TODO: each load of the linkage-table base into RAX should be replaced
+  ;; by a 4-byte NOP (#x0f #x1f #x40 #x00) whenever we replace the JMP/CALL.
   (let* ((linkage-cells (linkage-space-cells (core-linkage-space-info core)))
          (indices (coerce indices 'vector))
          (values (map 'vector (lambda (x) (aref linkage-cells (car x)))
@@ -1658,45 +1662,71 @@
                          :direction :io :if-exists :overwrite)
     (let* ((core-header (make-array +backend-page-bytes+ :element-type '(unsigned-byte 8)))
            (core-offset (read-core-header stream core-header))
-           (parsed-header (parse-core-header stream core-header))
+           (parsed-header (parse-core-header stream core-header core-offset))
            (space-list (core-header-space-list parsed-header)))
       (with-mapped-core (sap core-offset (core-header-total-npages parsed-header) stream)
         (let* ((spacemap (cons sap (sort (copy-list space-list) #'> :key #'space-addr)))
                (core (make-core spacemap (make-bounds 0 0) (make-bounds 0 0)
-                                :linkage-space-info (core-header-linkage-space-info parsed-header))))
-          #-immobile-space
-          (let* ((text-space (get-space immobile-text-core-space-id spacemap))
-                 (offsets-vector (%make-lisp-obj (logior (sap-int (space-physaddr text-space spacemap))
-                                                         lowtag-mask))))
-            (assert text-space)
-            (patch-asm-codeblob core)
-            ;; offset 0 is the offset of the ASM routine codeblob which was already processed.
-            (loop for j from 1 below (length offsets-vector)
+                                :linkage-space-info (core-header-linkage-space-info parsed-header)))
+               (features (detect-target-features spacemap)))
+          (cond
+            ((not (member :immobile-space features))
+             (let* ((text-space (get-space immobile-text-core-space-id spacemap))
+                    (offsets-vector (%make-lisp-obj (logior (sap-int (space-physaddr text-space spacemap))
+                                                            lowtag-mask))))
+               (assert text-space)
+               (patch-asm-codeblob core)
+               ;; offset 0 is the offset of the ASM routine codeblob which was already processed.
+               (loop for j from 1 below (length offsets-vector)
                   do (let ((vaddr (+ (space-addr text-space) (aref offsets-vector j)))
                            (physobj (%make-lisp-obj
                                      (logior (sap-int (sap+ (space-physaddr text-space spacemap)
                                                             (aref offsets-vector j)))
                                              other-pointer-lowtag))))
-                       (bypass-indirection-cells physobj vaddr core))))
-          #+immobile-space
-          (let* ((text-space (get-space immobile-text-core-space-id spacemap))
-                 (delta (- (translate-ptr (space-addr text-space) spacemap)
-                           (space-addr text-space))))
-            (walk-target-space (lambda (obj widetag size
+                       (bypass-indirection-cells physobj vaddr core)))))
+            (t
+             (let* ((text-space (get-space immobile-text-core-space-id spacemap))
+                    (delta (- (translate-ptr (space-addr text-space) spacemap)
+                              (space-addr text-space))))
+               (walk-target-space (lambda (obj widetag size
                                         &aux (vaddr (- (get-lisp-obj-address obj)
                                                        other-pointer-lowtag delta)))
                                  (declare (ignore widetag size))
                                  (bypass-indirection-cells obj vaddr core))
-                               immobile-text-core-space-id spacemap))
+                               immobile-text-core-space-id spacemap))))
           (persist-to-file spacemap core-offset stream))))))
 
 (defun split-core (input-pathname asm-pathname &rest args)
-  (let ((tmp (format nil "/tmp/sbcl~D.tmpcore" (sb-unix:unix-getpid))))
-    ;; input core could be readonly
-    (unwind-protect (progn (run-program "cp" `("--no-preserve=mode" ,input-pathname ,tmp)
-                                        :search t)
-                           #+x86-64 (redirect-text-space-calls tmp)
-                           (apply #'really-split-core tmp asm-pathname args))
+  (let ((tmp (flet ((try-directory (dir)
+                      (when (and dir (string/= "" dir) (sb-unix:unix-access dir sb-unix:w_ok))
+                        dir)))
+               (format nil "~a/sbcl~D.tmpcore"
+                       (or (try-directory (posix-getenv "TMPDIR"))
+                           (try-directory "/tmp")
+                           (try-directory (user-homedir-pathname))
+                           (error "Can't find a writeable directory for our split core."))
+                       (sb-unix:unix-getpid)))))
+    (with-open-file (stream input-pathname :element-type '(unsigned-byte 8))
+      (let* ((core-header (make-array +backend-page-bytes+ :element-type '(unsigned-byte 8)))
+             (core-offset (read-core-header stream core-header)))
+        (parse-core-header stream core-header core-offset)))
+    (unwind-protect
+         (progn
+           (ecase *heap-arrangement*
+             (:gencgc
+              ;; input core could be readonly
+              (run-program "cp" `("--no-preserve=mode" ,input-pathname ,tmp)
+                           :search t))
+             (:mark-region-gc
+              ;; Assume that the free space in the core hasn't been squashed out yet.
+              ;; I'm not sure which of these steps can operate in-place,
+              ;; so use an intermediate temp file for the reorg.
+              (let ((other-temp (concatenate 'string tmp "0")))
+                (reorganize-core input-pathname other-temp)
+                (move-dynamic-code-to-text-space other-temp tmp)
+                (delete-file other-temp))))
+           #+x86-64 (redirect-text-space-calls tmp)
+           (apply #'really-split-core tmp asm-pathname args))
       (delete-file tmp))))
 
 (defun cl-user::elfinate (&optional (args (cdr *posix-argv*)))
